# -*- coding: utf-8 -*-

# Copyright (C) 2014-2018 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#

from __future__ import absolute_import
from __future__ import unicode_literals

import dnf.automatic.main

import tests.support


FILE = tests.support.resource_path('etc/automatic.conf')


class TestConfig(tests.support.TestCase):
    def test_load(self):
        # test values from config file take effect if no overrides
        # note: config file specifies download = no apply = yes,
        # test expects implication to turn download into True
        conf = dnf.automatic.main.AutomaticConfig(FILE)
        self.assertTrue(conf.commands.apply_updates)
        self.assertTrue(conf.commands.download_updates)
        self.assertEqual(conf.commands.random_sleep, 300)
        self.assertEqual(conf.email.email_from, 'staring@crowd.net')

        # test overriding installupdates
        conf = dnf.automatic.main.AutomaticConfig(FILE, installupdates=False)
        # as per above, download is set false in config
        self.assertFalse(conf.commands.download_updates)
        self.assertFalse(conf.commands.apply_updates)

        # test overriding installupdates and downloadupdates
        conf = dnf.automatic.main.AutomaticConfig(FILE, downloadupdates=True, installupdates=False)
        self.assertTrue(conf.commands.download_updates)
        self.assertFalse(conf.commands.apply_updates)



# -*- coding: utf-8 -*-


from __future__ import absolute_import
from __future__ import unicode_literals

import dnf
import dnf.cli
import dnf.exceptions

from .common import TestCase


class DnfCliInitApiTest(TestCase):
    def test_cli_error(self):
        # dnf.cli.CliError
        self.assertHasAttr(dnf.cli, "CliError")
        ex = dnf.cli.CliError(value=None)
        self.assertHasType(ex, dnf.exceptions.Error)

    def test_cli(self):
        # dnf.cli.Cli
        self.assertHasAttr(dnf.cli, "Cli")
        self.assertHasType(dnf.cli.Cli, object)

    def test_command(self):
        # dnf.cli.Command
        self.assertHasAttr(dnf.cli, "Command")
        self.assertHasType(dnf.cli.Command, object)



# -*- coding: utf-8 -*-

# Copyright (C) 2014-2018 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#

from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals


import dnf.cli.commands.repoquery
import dnf.exceptions

import tests.support
from tests.support import mock


EXPECTED_INFO_FORMAT = """\
Name        : foobar
Version     : 1.0.1
Release     : 1.f20
Architecture: x86_64
Size        : 100
License     : BSD
Source RPM  : foo-1.0.1-1.f20.src.rpm
Build Date  : 1970-01-01 00:02
Packager    : Eastford
URL         : foorl.net
Summary     : it.
Description :
A desc.A desc.A desc.A desc.A desc.A desc.A desc.A desc.\n"""

EXPECTED_FILELIST_FORMAT = """\
/tmp/foobar
/var/foobar\
"""

EXPECTED_SOURCERPM_FORMAT = """\
foo-1.0.1-1.f20.src.rpm"""


class PkgStub(object):
    def __init__(self):
        self.arch = 'x86_64'
        self.buildtime = 120
        self.description = 'A desc.' * 8
        self.license = 'BSD'
        self.name = 'foobar'
        self.packager = 'Eastford'
        self.release = '1.f20'
        self.reponame = '@System'
        self._size = 100
        self.sourcerpm = 'foo-1.0.1-1.f20.src.rpm'
        self.summary = 'it.'
        self.url = 'foorl.net'
        self.version = '1.0.1'
        self.files = ['/tmp/foobar', '/var/foobar']


class ArgParseTest(tests.support.TestCase):
    def setUp(self):
        self.cmd = dnf.cli.commands.repoquery.RepoQueryCommand(
            tests.support.CliStub(tests.support.BaseCliStub()))

    def test_parse(self):
        tests.support.command_configure(self.cmd, ['--whatrequires', 'prudence'])
        self.assertEqual(self.cmd.opts.whatprovides, [])
        self.assertEqual(self.cmd.opts.whatrequires, ['prudence'])
        self.assertEqual(self.cmd.opts.queryformat,
                         dnf.cli.commands.repoquery.QFORMAT_DEFAULT)

    @mock.patch('argparse.ArgumentParser.print_help', lambda x: x)
    def test_conflict(self):
        with self.assertRaises(SystemExit) as sysexit, \
                tests.support.patch_std_streams() as (stdout, stderr):
                tests.support.command_configure(self.cmd, ['--conflicts', '%{name}', '--provides'])
        self.assertEqual(sysexit.exception.code, 1)

    def test_options(self):
        for arg in ('conflicts', 'enhances', 'provides',
                    'recommends', 'requires', 'suggests', 'supplements'):
            tests.support.command_configure(self.cmd, ['--' + arg])
            self.assertEqual(self.cmd.opts.packageatr, arg)

    def test_file(self):
        tests.support.command_configure(self.cmd, ['/var/foobar'])
        self.assertIsNone(self.cmd.opts.file)


class FilelistFormatTest(tests.support.TestCase):
    def test_filelist(self):
        self.cmd = dnf.cli.commands.repoquery.RepoQueryCommand(
            tests.support.CliStub(tests.support.BaseCliStub()))
        tests.support.command_configure(self.cmd, ['-l'])
        pkg = dnf.cli.commands.repoquery.PackageWrapper(PkgStub())
        self.assertEqual(self.cmd.build_format_fn(self.cmd.opts, pkg),
                         EXPECTED_FILELIST_FORMAT)


class SourceRPMFormatTest(tests.support.TestCase):
    def test_info(self):
        self.cmd = dnf.cli.commands.repoquery.RepoQueryCommand(
            tests.support.CliStub(tests.support.BaseCliStub()))
        tests.support.command_configure(self.cmd, ['--source'])
        pkg = dnf.cli.commands.repoquery.PackageWrapper(PkgStub())
        self.assertEqual(self.cmd.build_format_fn(self.cmd.opts, pkg),
                         EXPECTED_SOURCERPM_FORMAT)


class OutputTest(tests.support.TestCase):
    def test_output(self):
        pkg = PkgStub()
        fmt = dnf.cli.commands.repoquery.rpm2py_format(
            '%{name}-%{version}-%{release}.%{arch} (%{reponame})')
        self.assertEqual(fmt.format(pkg), 'foobar-1.0.1-1.f20.x86_64 (@System)')

    def test_illegal_attr(self):
        pkg = PkgStub()
        with self.assertRaises(AttributeError) as ctx:
            dnf.cli.commands.repoquery.rpm2py_format('%{notfound}').format(pkg)
        self.assertEqual(str(ctx.exception),
                         "'PkgStub' object has no attribute 'notfound'")


class Rpm2PyFormatTest(tests.support.TestCase):
    def test_rpm2py_format(self):
        fmt = dnf.cli.commands.repoquery.rpm2py_format('%{name}')
        self.assertEqual(fmt, '{0.name}')
        fmt = dnf.cli.commands.repoquery.rpm2py_format('%40{name}')
        self.assertEqual(fmt, '{0.name:<40}')
        fmt = dnf.cli.commands.repoquery.rpm2py_format('%-40{name}')
        self.assertEqual(fmt, '{0.name:>40}')
        fmt = dnf.cli.commands.repoquery.rpm2py_format(
            '%{name}-%{repoid} :: %-40{arch}')
        self.assertEqual(fmt, '{0.name}-{0.repoid} :: {0.arch:>40}')



# -*- coding: utf-8 -*-


from __future__ import absolute_import
from __future__ import unicode_literals

import dnf

from .common import TestCase


class DnfRPMTransactionApiTest(TestCase):
    def setUp(self):
        self.base = dnf.Base(dnf.conf.Conf())
        self.base.fill_sack(False, False)
        self.base.resolve()
        self.rpmTrans = self.base.transaction

    def tearDown(self):
        self.base.close()

    def test_iterator(self):
        # RPMTransaction.__iter__
        self.assertHasAttr(self.rpmTrans, "__iter__")
        for i in self.rpmTrans:
            pass

    def test_install_set(self):
        # RPMTransaction.install_set
        self.assertHasAttr(self.rpmTrans, "install_set")
        self.assertHasType(self.rpmTrans.install_set, set)

    def test_remove_set(self):
        # RPMTransaction.remove_set
        self.assertHasAttr(self.rpmTrans, "remove_set")
        self.assertHasType(self.rpmTrans.remove_set, set)



# -*- coding: utf-8 -*-


from __future__ import absolute_import
from __future__ import unicode_literals

import dnf

from .common import TestCase


class DnfConfTest(TestCase):
    def setUp(self):
        self.base = dnf.Base(dnf.conf.Conf())
        self.conf = self.base.conf

    def tearDown(self):
        self.base.close()

    def test_priorities(self):
        self.assertHasAttr(dnf.conf.config, "PRIO_EMPTY")
        self.assertHasType(dnf.conf.config.PRIO_EMPTY, int)

        self.assertHasAttr(dnf.conf.config, "PRIO_DEFAULT")
        self.assertHasType(dnf.conf.config.PRIO_DEFAULT, int)

        self.assertHasAttr(dnf.conf.config, "PRIO_MAINCONFIG")
        self.assertHasType(dnf.conf.config.PRIO_MAINCONFIG, int)

        self.assertHasAttr(dnf.conf.config, "PRIO_AUTOMATICCONFIG")
        self.assertHasType(dnf.conf.config.PRIO_AUTOMATICCONFIG, int)

        self.assertHasAttr(dnf.conf.config, "PRIO_REPOCONFIG")
        self.assertHasType(dnf.conf.config.PRIO_REPOCONFIG, int)

        self.assertHasAttr(dnf.conf.config, "PRIO_PLUGINDEFAULT")
        self.assertHasType(dnf.conf.config.PRIO_PLUGINDEFAULT, int)

        self.assertHasAttr(dnf.conf.config, "PRIO_PLUGINCONFIG")
        self.assertHasType(dnf.conf.config.PRIO_PLUGINCONFIG, int)

        self.assertHasAttr(dnf.conf.config, "PRIO_COMMANDLINE")
        self.assertHasType(dnf.conf.config.PRIO_COMMANDLINE, int)

        self.assertHasAttr(dnf.conf.config, "PRIO_RUNTIME")
        self.assertHasType(dnf.conf.config.PRIO_RUNTIME, int)

    def test_get_reposdir(self):
        # Conf.get_reposdir
        self.conf.reposdir = ["."]
        self.assertHasAttr(self.conf, "get_reposdir")
        self.assertHasType(self.conf.get_reposdir, str)

    def test_substitutions(self):
        # Conf.substitutions
        self.assertHasAttr(self.conf, "substitutions")
        self.assertHasType(self.conf.substitutions, dnf.conf.substitutions.Substitutions)

    def test_tempfiles(self):
        # Conf.tempfiles
        self.assertHasAttr(self.conf, "tempfiles")
        self.assertHasType(self.conf.tempfiles, list)

    def test_exclude_pkgs(self):
        # Conf.exclude_pkgs
        self.assertHasAttr(self.conf, "exclude_pkgs")
        self.conf.exclude_pkgs(pkgs=["package_a", "package_b"])

    def test_prepend_installroot(self):
        # Conf.prepend_installroot
        self.assertHasAttr(self.conf, "prepend_installroot")
        self.conf.prepend_installroot(optname="logdir")

    def test_read(self):
        # Conf.read
        self.assertHasAttr(self.conf, "read")
        self.conf.read(filename=None, priority=dnf.conf.config.PRIO_DEFAULT)

    def test_dump(self):
        # Conf.dump
        self.assertHasAttr(self.conf, "dump")
        self.assertHasType(self.conf.dump(), str)

    def test_releasever(self):
        # Conf.releasever
        self.assertHasAttr(self.conf, "releasever")
        self.conf.releasever = "test setter"
        self.assertHasType(self.conf.releasever, str)

    def test_arch(self):
        # Conf.arch
        self.assertHasAttr(self.conf, "arch")
        self.conf.arch = "aarch64"
        self.assertHasType(self.conf.arch, str)

    def test_basearch(self):
        # Conf.basearch
        self.assertHasAttr(self.conf, "basearch")
        self.conf.basearch = "aarch64"
        self.assertHasType(self.conf.basearch, str)

    def test_write_raw_configfile(self):
        # Conf.write_raw_configfile
        self.assertHasAttr(self.conf, "write_raw_configfile")
        s = dnf.conf.substitutions.Substitutions()
        self.conf.write_raw_configfile(filename="file.conf", section_id='main', substitutions=s, modify={})


class DnfSubstitutionsTest(TestCase):
    def test_update_from_etc(self):
        # Substitutions.update_from_etc
        substitutions = dnf.conf.substitutions.Substitutions()
        self.assertHasAttr(substitutions, "update_from_etc")
        substitutions.update_from_etc(installroot="path", varsdir=("/etc/path/", "/etc/path2"))



# -*- coding: utf-8 -*-

# Copyright (C) 2012-2018 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#

from __future__ import absolute_import
from __future__ import unicode_literals

import dnf.query
import dnf.subject

import tests.support


class QueriesTest(tests.support.TestCase):

    def setUp(self):
        self.sack = tests.support.mock_sack('main', 'updates')

    def test_duplicities(self):
        pepper = self.sack.query().installed().filter(name="pepper")
        # make sure 'pepper' package exists:
        self.assertEqual(len(pepper), 1)
        # we shouldn't see it more than once with a tricky query below:
        res = self.sack.query().installed().filter(name=["pep*", "*per"])
        res_set = set(res)
        self.assertEqual(len(res), len(res_set))

    def test_by_file(self):
        # check sanity first:
        q = self.sack.query().filter(file__eq="/raised/smile")
        self.assertEqual(len(q.run()), 1)
        q[0]

    def test_by_repo(self):
        pkgs = self.sack.query().filter(reponame__eq="updates")
        self.assertEqual(len(pkgs), tests.support.UPDATES_NSOLVABLES)
        pkgs = self.sack.query().filter(reponame__eq="main")
        self.assertEqual(len(pkgs), tests.support.MAIN_NSOLVABLES)

    def test_duplicated(self):
        pkgs = self.sack.query().duplicated()
        self.assertEqual(len(pkgs), 3)

    def test_extras(self):
        pkgs = self.sack.query().extras()
        self.assertEqual(len(pkgs), tests.support.TOTAL_RPMDB_COUNT - 4)

    def test_installed_exact(self):
        pkgs = self.sack.query().installed()._nevra("tour-4.9-0.noarch")
        self.assertEqual(len(pkgs), 0)
        pkgs = self.sack.query().installed()._nevra("tour-5-0.x86_64")
        self.assertEqual(len(pkgs), 0)
        pkgs = self.sack.query().installed()._nevra("tour-5-0.noarch")
        self.assertEqual(len(pkgs), 1)

    def test_latest(self):
        tours = self.sack.query().filter(name="tour")
        all_tours = sorted(tours.run(), reverse=True)
        head2 = all_tours[0:2]
        tail2 = all_tours[2:]
        pkgs = tours.latest(2).run()
        self.assertEqual(pkgs, head2)
        pkgs = tours.latest(-2).run()
        self.assertEqual(pkgs, tail2)


class SubjectTest(tests.support.TestCase):

    def setUp(self):
        self.sack = tests.support.mock_sack('main', 'updates')

    def test_wrong_name(self):
        subj = dnf.subject.Subject("call-his-wife-in")
        self.assertLength(subj.get_best_query(self.sack), 0)

    def test_query_composing(self):
        q = dnf.subject.Subject("librita").get_best_query(self.sack)
        q = q.filter(arch="i686")
        self.assertEqual(str(q[0]), "librita-1-1.i686")

    def test_icase_name(self):
        subj = dnf.subject.Subject("PEpper", ignore_case=True)
        q = subj.get_best_query(self.sack)
        self.assertLength(q, 4)

    def test_get_best_selector(self):
        s = dnf.subject.Subject("pepper-20-0.x86_64").get_best_selector(self.sack)
        self.assertIsNotNone(s)

    def test_get_best_selector_for_provides_glob(self):
        s = dnf.subject.Subject("*otus.so*").get_best_selector(self.sack)
        self.assertIsNotNone(s)

    def test_best_selector_for_version(self):
        sltr = dnf.subject.Subject("hole-2").get_best_selector(self.sack)
        self.assertCountEqual(map(str, sltr.matches()),
                              ['hole-2-1.x86_64', 'hole-2-1.i686'])

    def test_with_confusing_dashes(self):
        sltr = dnf.subject.Subject("mrkite-k-h").get_best_selector(self.sack)
        self.assertLength(sltr.matches(), 1)
        sltr = dnf.subject.Subject("mrkite-k-h.x86_64").get_best_selector(self.sack)
        self.assertLength(sltr.matches(), 1)


class DictsTest(tests.support.TestCase):

    def setUp(self):
        self.sack = tests.support.mock_sack('main', 'updates')

    def test_per_nevra_dict(self):
        pkgs = self.sack.query().filter(name="lotus")
        dct = dnf.query._per_nevra_dict(pkgs)
        self.assertCountEqual(dct.keys(),
                              ["lotus-3-16.x86_64", "lotus-3-16.i686"])
        self.assertCountEqual(dct.values(), pkgs)



# -*- coding: utf-8 -*-


from __future__ import absolute_import
from __future__ import unicode_literals

import dnf

from .common import TestCase


class DnfSackApiTest(TestCase):
    def setUp(self):
        self.base = dnf.Base(dnf.conf.Conf())

    def tearDown(self):
        self.base.close()

    def test_rpmdb_sack(self):
        # dnf.sack.rpmdb_sack
        self.assertHasAttr(dnf.sack, "rpmdb_sack")
        self.assertHasType(dnf.sack.rpmdb_sack(self.base), dnf.sack.Sack)

    def test_query(self):
        # Sack.query
        self.base.fill_sack(False, False)
        self.assertHasAttr(self.base.sack, "query")
        self.assertHasType(self.base.sack.query(flags=0), dnf.query.Query)



# -*- coding: utf-8 -*-

# Copyright (C) 2014-2018 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#

from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import datetime
import itertools
import shutil
import tempfile

import hawkey

import dnf.pycomp
import dnf.cli.commands.updateinfo

import tests.support
from tests.support import mock


class UpdateInfoCommandTest(tests.support.DnfBaseTestCase):

    """Test case validating updateinfo commands."""

    REPOS = []
    CLI = "mock"

    def setUp(self):
        """Prepare the test fixture."""
        super(UpdateInfoCommandTest, self).setUp()
        cachedir = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, cachedir)
        self.cli.base.conf.cachedir = cachedir
        self.cli.base.add_test_dir_repo('rpm', self.cli.base.conf)
        self._stdout = dnf.pycomp.StringIO()
        self.addCleanup(mock.patch.stopall)
        mock.patch(
            'dnf.cli.commands.updateinfo._',
            dnf.pycomp.NullTranslations().ugettext).start()
        mock.patch(
            'dnf.cli.commands.updateinfo.print',
            self._stub_print, create=True).start()

    def _stub_print(self, *objects):
        """Pretend to print to standard output."""
        print(*objects, file=self._stdout)

    def test_avail_filter_pkgs(self):
        """Test querying with a packages filter."""
        cmd = dnf.cli.commands.updateinfo.UpdateInfoCommand(self.cli)
        tests.support.command_configure(cmd, [])
        apkg_adv_insts = cmd.available_apkg_adv_insts(['to*r', 'nxst'])
        self.assertCountEqual(
            ((apk.filename, adv.id, ins) for apk, adv, ins in apkg_adv_insts),
            [('tour-5-1.noarch.rpm', 'DNF-2014-3', False)],
            'incorrect pairs')

    def test_avail_filter_pkgs_nonex(self):
        """Test querying with a non-existent packages filter."""
        cmd = dnf.cli.commands.updateinfo.UpdateInfoCommand(self.cli)
        tests.support.command_configure(cmd, ['non-existent'])
        apkg_adv_insts = cmd.available_apkg_adv_insts(cmd.opts.spec)
        self.assertCountEqual(
            ((apk.filename, adv.id, ins) for apk, adv, ins in apkg_adv_insts),
            [], 'incorrect pairs')

    def test_avail_filter_security(self):
        """Test querying with a security filter."""
        cmd = dnf.cli.commands.updateinfo.UpdateInfoCommand(self.cli)
        tests.support.command_configure(cmd, [])
        apkg_adv_insts = cmd.available_apkg_adv_insts(['security'])
        self.assertCountEqual(
            ((apk.filename, adv.id, ins) for apk, adv, ins in apkg_adv_insts),
            [('tour-5-1.noarch.rpm', 'DNF-2014-3', False)],
            'incorrect pairs')

    def test_inst(self):
        """Test installed triplets querying."""
        cmd = dnf.cli.commands.updateinfo.UpdateInfoCommand(self.cli)
        tests.support.command_configure(cmd, [])
        apkg_adv_insts = cmd.installed_apkg_adv_insts([])
        self.assertCountEqual(
            ((apk.filename, adv.id, ins) for apk, adv, ins in apkg_adv_insts),
            [('tour-4-4.noarch.rpm', 'DNF-2014-1', True),
             ('tour-5-0.noarch.rpm', 'DNF-2014-2', True)],
            'incorrect pairs')

    def test_inst_filter_bugfix(self):
        """Test querying with a bugfix filter."""
        cmd = dnf.cli.commands.updateinfo.UpdateInfoCommand(self.cli)
        tests.support.command_configure(cmd, ['bugfix'])
        apkg_adv_insts = cmd.installed_apkg_adv_insts(cmd.opts.spec)
        self.assertCountEqual(
            ((apk.filename, adv.id, ins) for apk, adv, ins in apkg_adv_insts),
            [('tour-4-4.noarch.rpm', 'DNF-2014-1', True)],
            'incorrect pairs')

    def test_inst_filter_enhancement(self):
        """Test querying with an enhancement filter."""
        cmd = dnf.cli.commands.updateinfo.UpdateInfoCommand(self.cli)
        tests.support.command_configure(cmd, ['enhancement'])
        apkg_adv_insts = cmd.installed_apkg_adv_insts(cmd.opts.spec)
        self.assertCountEqual(
            ((apk.filename, adv.id, ins) for apk, adv, ins in apkg_adv_insts),
            [('tour-5-0.noarch.rpm', 'DNF-2014-2', True)],
            'incorrect pairs')

    def test_upd(self):
        """Test updating triplets querying."""
        cmd = dnf.cli.commands.updateinfo.UpdateInfoCommand(self.cli)
        tests.support.command_configure(cmd, [])
        apkg_adv_insts = cmd.updating_apkg_adv_insts([])
        self.assertCountEqual(
            ((apk.filename, adv.id, ins) for apk, adv, ins in apkg_adv_insts),
            [('tour-5-1.noarch.rpm', 'DNF-2014-3', False)],
            'incorrect pairs')

    def test_all(self):
        """Test all triplets querying."""
        cmd = dnf.cli.commands.updateinfo.UpdateInfoCommand(self.cli)
        tests.support.command_configure(cmd, [])
        apkg_adv_insts = cmd.all_apkg_adv_insts([])
        self.assertCountEqual(
            ((apk.filename, adv.id, ins) for apk, adv, ins in apkg_adv_insts),
            [('tour-4-4.noarch.rpm', 'DNF-2014-1', True),
             ('tour-5-0.noarch.rpm', 'DNF-2014-2', True),
             ('tour-5-1.noarch.rpm', 'DNF-2014-3', False)])

    def test_all_filter_advisories(self):
        """Test querying with an advisories filter."""
        cmd = dnf.cli.commands.updateinfo.UpdateInfoCommand(self.cli)
        tests.support.command_configure(cmd, ['DNF-201*-[13]', 'NO-0000-0'])
        apkg_adv_insts = cmd.all_apkg_adv_insts(cmd.opts.spec)
        self.assertCountEqual(
            ((apk.filename, adv.id, ins) for apk, adv, ins in apkg_adv_insts),
            [('tour-4-4.noarch.rpm', 'DNF-2014-1', True),
             ('tour-5-1.noarch.rpm', 'DNF-2014-3', False)],
            'incorrect pairs')

    def test_display_list_mixed(self):
        """Test list displaying with mixed installs."""
        apkg_adv_insts = itertools.chain(
            ((apkg, adv, False)
             for pkg in self.cli.base.sack.query().installed()
             for adv in pkg.get_advisories(hawkey.GT)
             for apkg in adv.packages),
            ((apkg, adv, True)
             for pkg in self.cli.base.sack.query().installed()
             for adv in pkg.get_advisories(hawkey.LT | hawkey.EQ)
             for apkg in adv.packages))
        cmd = dnf.cli.commands.updateinfo.UpdateInfoCommand(self.cli)
        tests.support.command_configure(cmd, ['--all'])
        cmd.display_list(apkg_adv_insts)
        self.assertEqual(
            self._stdout.getvalue(),
            'i DNF-2014-1 bugfix       tour-4-4.noarch\n'
            'i DNF-2014-2 enhancement  tour-5-0.noarch\n'
            '  DNF-2014-3 Unknown/Sec. tour-5-1.noarch\n',
            'incorrect output'
        )

    def test_display_info_verbose(self):
        """Test verbose displaying."""
        apkg_adv_insts = (
            (apkg, adv, False) for pkg in self.cli.base.sack.query().installed()
            for adv in pkg.get_advisories(hawkey.GT)
            for apkg in adv.packages
        )
        self.cli.base.set_debuglevel(dnf.const.VERBOSE_LEVEL)
        cmd = dnf.cli.commands.updateinfo.UpdateInfoCommand(self.cli)
        tests.support.command_configure(cmd, [])
        cmd.display_info(apkg_adv_insts)
        updated = datetime.datetime.fromtimestamp(1404841143)
        self.assertEqual(self._stdout.getvalue(),
                         '========================================'
                         '=======================================\n'
                         '  tour-5-1\n'
                         '========================================'
                         '=======================================\n'
                         '  Update ID: DNF-2014-3\n'
                         '       Type: security\n'
                         '    Updated: ' + str(updated) + '\n'
                         'Description: testing advisory\n'
                         '      Files: tour-5-1.noarch.rpm\n',
                         'incorrect output')

    def test_display_info_verbose_mixed(self):
        """Test verbose displaying with mixed installs."""
        apkg_adv_insts = itertools.chain(
            ((apkg, adv, False)
             for pkg in self.cli.base.sack.query().installed()
             for adv in pkg.get_advisories(hawkey.GT)
             for apkg in adv.packages),
            ((apkg, adv, True)
             for pkg in self.cli.base.sack.query().installed()
             for adv in pkg.get_advisories(hawkey.LT | hawkey.EQ)
             for apkg in adv.packages))
        self.cli.base.set_debuglevel(dnf.const.VERBOSE_LEVEL)
        cmd = dnf.cli.commands.updateinfo.UpdateInfoCommand(self.cli)
        tests.support.command_configure(cmd, ['--all'])
        cmd.display_info(apkg_adv_insts)
        updated1 = datetime.datetime.fromtimestamp(1404840841)
        updated2 = datetime.datetime.fromtimestamp(1404841082)
        updated3 = datetime.datetime.fromtimestamp(1404841143)
        self.assertEqual(self._stdout.getvalue(),
                         '========================================'
                         '=======================================\n'
                         '  tour-4-4\n'
                         '========================================'
                         '=======================================\n'
                         '  Update ID: DNF-2014-1\n'
                         '       Type: bugfix\n'
                         '    Updated: ' + str(updated1) + '\n'
                         'Description: testing advisory\n'
                         '      Files: tour-4-4.noarch.rpm\n'
                         '  Installed: true\n'
                         '\n'
                         '========================================'
                         '=======================================\n'
                         '  tour-5-0\n'
                         '========================================'
                         '=======================================\n'
                         '  Update ID: DNF-2014-2\n'
                         '       Type: enhancement\n'
                         '    Updated: ' + str(updated2) + '\n'
                         'Description: testing advisory\n'
                         '      Files: tour-5-0.noarch.rpm\n'
                         '  Installed: true\n'
                         '\n'
                         '========================================'
                         '=======================================\n'
                         '  tour-5-1\n'
                         '========================================'
                         '=======================================\n'
                         '  Update ID: DNF-2014-3\n'
                         '       Type: security\n'
                         '    Updated: ' + str(updated3) + '\n'
                         'Description: testing advisory\n'
                         '      Files: tour-5-1.noarch.rpm\n'
                         '  Installed: false\n',
                         'incorrect output')

    # This test also tests the display_summary and available_apkg_adv_insts
    # methods.
    def test_run_available(self):
        """Test running with available advisories."""
        cmd = dnf.cli.commands.updateinfo.UpdateInfoCommand(self.cli)
        tests.support.command_run(cmd, [])
        self.assertEqual(self._stdout.getvalue(),
                         'Updates Information Summary: available\n'
                         '    1 Security notice(s)\n'
                         '        1 Unknown Security notice(s)\n',
                         'incorrect output')

    # This test also tests the display_list and available_apkg_adv_insts
    # methods.
    def test_run_list(self):
        """Test running the list sub-command."""
        cmd = dnf.cli.commands.updateinfo.UpdateInfoCommand(self.cli)
        tests.support.command_run(cmd, ['list'])
        self.assertEqual(self._stdout.getvalue(),
                         'DNF-2014-3 Unknown/Sec. tour-5-1.noarch\n',
                         'incorrect output')

    # This test also tests the display_info and available_apkg_adv_insts
    # methods.
    def test_run_info(self):
        """Test running the info sub-command."""
        self.cli.base.set_debuglevel(2)
        cmd = dnf.cli.commands.updateinfo.UpdateInfoCommand(self.cli)
        tests.support.command_run(cmd, ['info'])
        updated = datetime.datetime.fromtimestamp(1404841143)
        self.assertEqual(self._stdout.getvalue(),
                         '========================================'
                         '=======================================\n'
                         '  tour-5-1\n'
                         '========================================'
                         '=======================================\n'
                         '  Update ID: DNF-2014-3\n'
                         '       Type: security\n'
                         '    Updated: ' + str(updated) + '\n'
                         'Description: testing advisory\n',
                         'incorrect output')



# -*- coding: utf-8 -*-

# Copyright (C) 2014-2018 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#

from __future__ import absolute_import
from __future__ import unicode_literals

import dnf.automatic.emitter

import tests.support
from tests.support import mock, mock_open


MSG = """\
downloaded on myhost:
packages..."""


class TestEmitter(tests.support.TestCase):
    def test_prepare_msg(self):
        emitter = dnf.automatic.emitter.Emitter('myhost')
        emitter.notify_available('packages...')
        emitter.notify_downloaded()
        with mock.patch('dnf.automatic.emitter.DOWNLOADED', 'downloaded on %s:'):
            self.assertEqual(emitter._prepare_msg(), MSG)


class TestMotdEmitter(tests.support.TestCase):
    def test_motd(self):
        m = mock_open()
        with mock.patch('dnf.automatic.emitter.open', m, create=True):
            emitter = dnf.automatic.emitter.MotdEmitter('myhost')
            emitter.notify_available('packages...')
            emitter.notify_downloaded()
            with mock.patch('dnf.automatic.emitter.DOWNLOADED', 'downloaded on %s:'):
                emitter.commit()
            handle = m()
            handle.write.assert_called_once_with(MSG)



# -*- coding: utf-8 -*-


from __future__ import absolute_import
from __future__ import unicode_literals

import dnf

from .common import TestCase


class DnfCallbackApiTest(TestCase):
    def test_pkg_downgrade(self):
        # dnf.callback.PKG_DOWNGRADE
        self.assertHasAttr(dnf.callback, "PKG_DOWNGRADE")
        self.assertHasType(dnf.callback.PKG_DOWNGRADE, int)

    def test_downgraded(self):
        # dnf.callback.PKG_DOWNGRADED
        self.assertHasAttr(dnf.callback, "PKG_DOWNGRADED")
        self.assertHasType(dnf.callback.PKG_DOWNGRADED, int)

    def test_pkg_install(self):
        # dnf.callback.PKG_INSTALL
        self.assertHasAttr(dnf.callback, "PKG_INSTALL")
        self.assertHasType(dnf.callback.PKG_INSTALL, int)

    def test_pkg_obsolete(self):
        # dnf.callback.PKG_OBSOLETE
        self.assertHasAttr(dnf.callback, "PKG_OBSOLETE")
        self.assertHasType(dnf.callback.PKG_OBSOLETE, int)

    def test_pkg_obsoleted(self):
        # dnf.callback.PKG_OBSOLETED
        self.assertHasAttr(dnf.callback, "PKG_OBSOLETED")
        self.assertHasType(dnf.callback.PKG_OBSOLETED, int)

    def test_pkg_reinstall(self):
        # dnf.callback.PKG_REINSTALL
        self.assertHasAttr(dnf.callback, "PKG_REINSTALL")
        self.assertHasType(dnf.callback.PKG_REINSTALL, int)

    def test_pkg_reinstalled(self):
        # dnf.callback.PKG_REINSTALLED
        self.assertHasAttr(dnf.callback, "PKG_REINSTALLED")
        self.assertHasType(dnf.callback.PKG_REINSTALLED, int)

    def test_pkg_remove(self):
        # dnf.callback.PKG_REMOVE
        self.assertHasAttr(dnf.callback, "PKG_REMOVE")
        self.assertHasType(dnf.callback.PKG_REMOVE, int)

    def test_pkg_upgrade(self):
        # dnf.callback.PKG_UPGRADE
        self.assertHasAttr(dnf.callback, "PKG_UPGRADE")
        self.assertHasType(dnf.callback.PKG_UPGRADE, int)

    def test_pkg_upgraded(self):
        # dnf.callback.PKG_UPGRADED
        self.assertHasAttr(dnf.callback, "PKG_UPGRADED")
        self.assertHasType(dnf.callback.PKG_UPGRADED, int)

    def test_pkg_cleanup(self):
        # dnf.callback.PKG_CLEANUP
        self.assertHasAttr(dnf.callback, "PKG_CLEANUP")
        self.assertHasType(dnf.callback.PKG_CLEANUP, int)

    def test_pkg_verify(self):
        # dnf.callback.PKG_VERIFY
        self.assertHasAttr(dnf.callback, "PKG_VERIFY")
        self.assertHasType(dnf.callback.PKG_VERIFY, int)

    def test_pkg_scriptlet(self):
        # dnf.callback.PKG_SCRIPTLET
        self.assertHasAttr(dnf.callback, "PKG_SCRIPTLET")
        self.assertHasType(dnf.callback.PKG_SCRIPTLET, int)

    def test_trans_preparation(self):
        # dnf.callback.TRANS_PREPARATION
        self.assertHasAttr(dnf.callback, "TRANS_PREPARATION")
        self.assertHasType(dnf.callback.TRANS_PREPARATION, int)

    def test_trans_post(self):
        # dnf.callback.TRANS_POST
        self.assertHasAttr(dnf.callback, "TRANS_POST")
        self.assertHasType(dnf.callback.TRANS_POST, int)

    def test_status_ok(self):
        # dnf.callback.STATUS_OK
        self.assertHasAttr(dnf.callback, "STATUS_OK")
        self.assertHasType(dnf.callback.STATUS_OK, object)

    def test_status_failed(self):
        # dnf.callback.STATUS_FAILED
        self.assertHasAttr(dnf.callback, "STATUS_FAILED")
        self.assertHasType(dnf.callback.STATUS_FAILED, int)

    def test_status_already_exists(self):
        # dnf.callback.STATUS_ALREADY_EXISTS
        self.assertHasAttr(dnf.callback, "STATUS_ALREADY_EXISTS")
        self.assertHasType(dnf.callback.STATUS_ALREADY_EXISTS, int)

    def test_status_mirror(self):
        # dnf.callback.STATUS_MIRROR
        self.assertHasAttr(dnf.callback, "STATUS_MIRROR")
        self.assertHasType(dnf.callback.STATUS_MIRROR, int)

    def test_status_drpm(self):
        # dnf.callback.STATUS_DRPM
        self.assertHasAttr(dnf.callback, "STATUS_DRPM")
        self.assertHasType(dnf.callback.STATUS_DRPM, int)

    def test_payload(self):
        # dnf.callback.Payload
        self.assertHasAttr(dnf.callback, "Payload")
        self.assertHasType(dnf.callback.Payload, object)

    def test_payload_init(self):
        # dnf.callback.Payload.__init__
        download_progress = dnf.callback.DownloadProgress()
        _ = dnf.callback.Payload(progress=download_progress)

    def test_payload_str(self):
        # dnf.callback.Payload.__str__
        download_progress = dnf.callback.DownloadProgress()
        payload = dnf.callback.Payload(progress=download_progress)
        payload.__str__()

    def test_payload_download_size(self):
        # dnf.callback.Payload.download_size
        download_progress = dnf.callback.DownloadProgress()
        payload = dnf.callback.Payload(progress=download_progress)
        self.assertHasAttr(payload, "download_size")
        self.assertHasType(payload.download_size, object)

    def test_download_progress(self):
        # dnf.callback.DownloadProgress
        self.assertHasAttr(dnf.callback, "DownloadProgress")
        self.assertHasType(dnf.callback.DownloadProgress, object)

    def test_download_progress_end(self):
        # dnf.callback.DownloadProgress.end
        download_progress = dnf.callback.DownloadProgress()
        payload = dnf.callback.Payload(progress=download_progress)
        download_progress.end(payload=payload, status=dnf.callback.STATUS_OK, msg="err_msg")

    def test_download_progress_progress(self):
        # dnf.callback.DownloadProgress.progress
        download_progress = dnf.callback.DownloadProgress()
        payload = dnf.callback.Payload(progress=download_progress)
        download_progress.progress(payload=payload, done=0)

    def test_download_progress_start(self):
        # dnf.callback.DownloadProgress.start
        download_progress = dnf.callback.DownloadProgress()
        download_progress.start(total_files=1, total_size=1, total_drpms=0)

    def test_TransactionProgress(self):
        # dnf.callback.TransactionProgress
        self.assertHasAttr(dnf.callback, "TransactionProgress")
        self.assertHasType(dnf.callback.TransactionProgress, object)



# -*- coding: utf-8 -*-

# Copyright (C) 2012-2018 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#

from __future__ import absolute_import
from __future__ import unicode_literals

import binascii
import hawkey
import rpm

from dnf.pycomp import long

import tests.support
from tests.support import mock


TOUR_MD5 = binascii.unhexlify("68e9ded8ea25137c964a638f12e9987c")
TOUR_SHA256 = binascii.unhexlify("ce77c1e5694b037b6687cf0ab812ca60431ec0b65116abbb7b82684f0b092d62")
TOUR_WRONG_MD5 = binascii.unhexlify("ffe9ded8ea25137c964a638f12e9987c")
TOUR_SIZE = 2317


class PackageTest(tests.support.DnfBaseTestCase):

    REPOS = ['main']

    def setUp(self):
        super(PackageTest, self).setUp()
        self.pkg = self.sack.query().available().filter(name="pepper")[1]

    def test_from_cmdline(self):
        self.sack.create_cmdline_repo()
        local_pkg = self.sack.add_cmdline_package(tests.support.TOUR_44_PKG_PATH)
        self.assertTrue(local_pkg._from_cmdline)
        self.assertFalse(self.pkg._from_cmdline)

    def test_from_system(self):
        pkg = self.sack.query().installed().filter(name="pepper")[0]
        self.assertTrue(pkg._from_system)
        self.assertFalse(self.pkg._from_system)

    def test_header(self):
        self.sack.create_cmdline_repo()
        pkg = self.sack.add_cmdline_package(tests.support.TOUR_44_PKG_PATH)
        header = pkg._header
        self.assertIsInstance(header, rpm.hdr)

        def fn_getter():
            return tests.support.NONEXISTENT_FILE

        with mock.patch.object(pkg, 'localPkg', fn_getter):
            with self.assertRaises(IOError):
                pkg._header

    @mock.patch("dnf.package.Package.rpmdbid", long(3))
    def test_idx(self):
        """ pkg.idx is an int. """
        pkg = self.sack.query().installed().filter(name="pepper")[0]
        self.assertEqual(type(pkg.idx), int)

    def test_pkgtup(self):
        self.assertEqual(self.pkg.pkgtup, ('pepper', 'x86_64', '0', '20', '0'))

    @mock.patch("dnf.package.Package.location", 'f/foo.rpm')
    def test_localPkg(self):
        self.pkg.repo.basecachedir = '/cachedir'
        self.pkg.repo.baseurl = ['file:///mnt/cd']
        self.assertTrue(self.pkg._is_local_pkg())
        self.assertEqual(self.pkg.localPkg(), '/mnt/cd/f/foo.rpm')
        self.pkg.repo.baseurl = ['http://remote']
        self.assertFalse(self.pkg._is_local_pkg())
        self.assertEqual(self.pkg.localPkg(),
                         self.pkg.repo._repo.getCachedir() + '/packages/foo.rpm')

    def test_verify(self):
        with mock.patch.object(self.pkg, 'localPkg',
                               return_value=tests.support.TOUR_44_PKG_PATH):
            self.pkg._chksum = (hawkey.CHKSUM_MD5, TOUR_MD5)
            self.pkg._size = TOUR_SIZE
            self.assertTrue(self.pkg.verifyLocalPkg())
            self.pkg._chksum = (hawkey.CHKSUM_MD5, TOUR_WRONG_MD5)
            self.assertFalse(self.pkg.verifyLocalPkg())

    def test_return_id_sum(self):
        self.pkg._chksum = (hawkey.CHKSUM_MD5, TOUR_MD5)
        self.assertEqual(self.pkg.returnIdSum(),
                         ('md5', '68e9ded8ea25137c964a638f12e9987c'))

    def test_verify_local(self):
        self.sack.create_cmdline_repo()
        local_pkg = self.sack.add_cmdline_package(tests.support.TOUR_44_PKG_PATH)
        self.assertEqual(local_pkg.reponame, hawkey.CMDLINE_REPO_NAME)
        self.assertTrue(local_pkg.verifyLocalPkg())

    def test_chksum_local(self):
        self.sack.create_cmdline_repo()
        local_pkg = self.sack.add_cmdline_package(tests.support.TOUR_44_PKG_PATH)
        chksum = local_pkg._chksum
        self.assertEqual(chksum[0], hawkey.CHKSUM_SHA256)
        self.assertEqual(chksum[1], TOUR_SHA256)

    def test_verify_installed(self):
        pkg = self.sack.query().installed().filter(name="pepper")[0]
        self.assertRaises(ValueError, pkg.verifyLocalPkg)



# -*- coding: utf-8 -*-

# Copyright (C) 2013-2018 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#

from __future__ import absolute_import
from __future__ import unicode_literals

import dnf.logging
import dnf.plugin
import dnf.pycomp

import tests.support


PLUGINS = "%s/tests/plugins" % tests.support.dnf_toplevel()


def testconf():
    conf = tests.support.FakeConf()
    conf.pluginpath = [PLUGINS]
    conf.pluginconfpath = [PLUGINS]
    return conf


class PluginTest(tests.support.TestCase):
    def setUp(self):
        self.plugins = dnf.plugin.Plugins()
        self.plugins._load(testconf(), (), ())

    def tearDown(self):
        self.plugins._unload()

    def test_load(self):
        self.assertLength(self.plugins.plugin_cls, 1)
        cls = self.plugins.plugin_cls[0]
        assert(issubclass(cls, dnf.plugin.Plugin))
        self.assertEqual(cls.name, 'lucky')

    def test_runs(self):
        self.assertLength(self.plugins.plugins, 0)
        self.plugins._run_init(None, None)
        self.assertLength(self.plugins.plugins, 1)
        self.plugins._run_config()
        lucky = self.plugins.plugins[0]
        self.assertTrue(lucky._config)

    def test_config(self):
        base = tests.support.MockBase()
        base.conf.pluginconfpath = ['/wrong', PLUGINS]
        self.plugins._run_init(base, None)
        lucky = self.plugins.plugins[0]
        conf = lucky.read_config(base.conf)
        self.assertTrue(conf.getboolean('main', 'enabled'))
        self.assertEqual(conf.get('main', 'wanted'), '/to/be/haunted')
        base.close()

    def test_disabled(self):
        base = tests.support.MockBase()
        base.conf.pluginconfpath = [PLUGINS]
        self.plugins._run_init(base, None)
        self.assertFalse(any([p.name == 'disabled-plugin'
                              for p in self.plugins.plugins]))
        self.assertLength(self.plugins.plugin_cls, 1)
        self.assertEqual(self.plugins.plugin_cls[0].name, 'lucky')
        base.close()


class PluginSkipsTest(tests.support.TestCase):
    def test_skip(self):
        self.plugins = dnf.plugin.Plugins()
        self.plugins._load(testconf(), ('luck*',), ())
        self.assertLength(self.plugins.plugin_cls, 0)

    def tearDown(self):
        self.plugins._unload()


class PluginNonExistentTest(tests.support.TestCase):

    """Tests with a non-existent plugin."""

    def test_logs_traceback(self):
        """Test whether the traceback is logged if a plugin cannot be imported."""
        package = dnf.pycomp.ModuleType('testpkg')
        package.__path__ = []
        stream = dnf.pycomp.StringIO()

        with tests.support.wiretap_logs('dnf', dnf.logging.SUBDEBUG, stream):
            dnf.plugin._import_modules(package, ('nonexistent.py',))

        end = ('Error: No module named \'testpkg\'\n' if dnf.pycomp.PY3
               else 'Error: No module named testpkg.nonexistent\n')
        self.assertTracebackIn(end, stream.getvalue())



# -*- coding: utf-8 -*-

# Copyright (C) 2015-2018 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#

from __future__ import absolute_import
from __future__ import unicode_literals

import dnf.exceptions

import tests.support


class SubjectTest(tests.support.DnfBaseTestCase):

    REPOS = ['main', 'third_party']

    def setUp(self):
        super(SubjectTest, self).setUp()
        pkg = self.base.sack.query().filter(name='lotus', arch='x86_64')[0]
        self.base.sack.add_excludes([pkg])

    def test_best_selectors_no_glob(self):
        subj = dnf.subject.Subject('pepper')
        sltrs = subj._get_best_selectors(self.base)
        self.assertEqual(len(sltrs), 1)

    def test_best_selectors_glob(self):
        subj = dnf.subject.Subject('l*')
        sltrs = subj._get_best_selectors(self.base)
        q = self.base.sack.query().filter(name__glob='l*')
        self.assertEqual(len(sltrs), len(set(map(lambda p: p.name, q))))

    def test_best_selectors_arch(self):
        subj = dnf.subject.Subject('l*.x86_64')
        sltrs = subj._get_best_selectors(self.base)
        q = self.base.sack.query().filter(name__glob='l*', arch__eq='x86_64')
        self.assertEqual(len(sltrs), len(set(map(lambda p: p.name, q))))
        for sltr in sltrs:
            for pkg in sltr.matches():
                self.assertEqual(pkg.arch, 'x86_64')

    def test_best_selectors_ver(self):
        subj = dnf.subject.Subject('*-1-1')
        sltrs = subj._get_best_selectors(self.base)
        for sltr in sltrs:
            for pkg in sltr.matches():
                self.assertEqual(pkg.evr, '1-1')



# -*- coding: utf-8 -*-

# Copyright (C) 2012-2018 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#

from __future__ import absolute_import
from __future__ import unicode_literals

import rpm

import dnf
import dnf.goal

import tests.support
from tests.support import mock


class DowngradeTest(tests.support.ResultTestCase):

    REPOS = ["main"]
    INIT_SACK = True

    @mock.patch('dnf.rpm.transaction.TransactionWrapper')
    def test_package_downgrade(self, ts):
        pkgs = self.base.add_remote_rpms([tests.support.TOUR_44_PKG_PATH])
        cnt = self.base.package_downgrade(pkgs[0])
        self.base._ts.setProbFilter.assert_called_with(
            rpm.RPMPROB_FILTER_OLDPACKAGE)
        self.assertGreater(cnt, 0)
        (installed, removed) = self.installed_removed(self.base)
        self.assertCountEqual(map(str, installed), ("tour-4-4.noarch", ))
        self.assertCountEqual(map(str, removed), ("tour-5-0.noarch", ))

    def test_downgrade(self):
        cnt = self.base.downgrade("tour")
        self.assertGreater(cnt, 0)

        new_pkg = self.base.sack.query().available().filter(name="tour")[0]
        self.assertEqual(new_pkg.evr, "4.6-1")
        new_set = tests.support.installed_but(self.base.sack, "tour") + [new_pkg]
        self.assertResult(self.base, new_set)

    def test_downgrade2(self):
        # override base with custom repos
        self.base = tests.support.MockBase("old_versions")
        self.base.downgrade("tour")
        installed, removed = self.installed_removed(self.base)
        self.assertCountEqual(map(str, installed), ['tour-4.9-1.noarch'])
        self.assertCountEqual(map(str, removed), ['tour-5-0.noarch'])


class DowngradeTest2(tests.support.DnfBaseTestCase):

    REPOS = ["main"]
    INIT_SACK = True

    def setUp(self):
        super(DowngradeTest2, self).setUp()
        self.base._goal = mock.create_autospec(dnf.goal.Goal)

    def test_downgrade_pkgnevra(self):
        """ Downgrade should handle full NEVRAs. """
        tests.support.ObjectMatcher(dnf.package.Package, {'name': 'tour'})
        with self.assertRaises(dnf.exceptions.PackagesNotInstalledError):
            self.base.downgrade('tour-0:5-0.noarch')

    def test_downgrade_notinstalled(self):
        pkg = tests.support.ObjectMatcher(dnf.package.Package, {'name': 'lotus'})

        with self.assertRaises(dnf.exceptions.PackagesNotInstalledError) as context:
            self.base.downgrade('lotus')
        self.assertEqual(context.exception.pkg_spec, 'lotus')
        self.assertEqual(tuple(context.exception.packages), (pkg,) * 2)
        self.assertEqual(self.goal.mock_calls, [])

    def test_downgrade_notfound(self):
        with self.assertRaises(dnf.exceptions.PackageNotFoundError) as context:
            self.base.downgrade('non-existent')
        self.assertEqual(context.exception.pkg_spec, 'non-existent')
        self.assertEqual(self.goal.mock_calls, [])

    def test_downgrade_nodowngrade(self):
        downgraded_count = self.base.downgrade('pepper')

        self.assertEqual(self.goal.mock_calls, [])
        self.assertEqual(downgraded_count, 0)



# -*- coding: utf-8 -*-


from __future__ import absolute_import
from __future__ import unicode_literals

import dnf

from .common import TestCase


class DnfPackageApiTest(TestCase):
    def setUp(self):
        self.base = dnf.Base(dnf.conf.Conf())
        self.package = self._get_pkg()

    def tearDown(self):
        self.base.close()

    def test_arch(self):
        # Package.arch
        self.assertHasAttr(self.package, "arch")
        self.assertHasType(self.package.arch, str)

    def test_baseurl(self):
        # Package.baseurl
        self.assertHasAttr(self.package, "baseurl")
        # The value is missing here so the type is type(None) but it should be str
        # self.assertHasType(self.package.baseurl, str)
        self.package.baseurl

    def test_buildtime(self):
        # Package.buildtime
        self.assertHasAttr(self.package, "buildtime")
        self.assertHasType(self.package.buildtime, int)

    def test_chksum(self):
        # Package.chksum()
        self.assertHasAttr(self.package, "chksum")
        # The value is missing here so the type is type(None) but it should be str
        # self.assertHasType(self.package.chksum, str)
        self.package.chksum

    def test_conflicts(self):
        # Package.conflicts()
        self.assertHasAttr(self.package, "conflicts")
        self.assertHasType(self.package.conflicts, list)

    def test_debug_name(self):
        # Package.debug_name
        self.assertHasAttr(self.package, "debug_name")
        self.assertHasType(self.package.debug_name, str)

    def test_description(self):
        # Package.description
        self.assertHasAttr(self.package, "description")
        self.assertHasType(self.package.description, str)

    def test_downloadsize(self):
        # Package.downloadsize
        self.assertHasAttr(self.package, "downloadsize")
        self.assertHasType(self.package.downloadsize, int)

    def test_epoch(self):
        # Package.epoch
        self.assertHasAttr(self.package, "epoch")
        self.assertHasType(self.package.epoch, int)

    def test_enhances(self):
        # Package.enhances
        self.assertHasAttr(self.package, "enhances")
        self.assertHasType(self.package.enhances, list)

    def test_evr(self):
        # Package.evr
        self.assertHasAttr(self.package, "evr")
        self.assertHasType(self.package.evr, str)

    def test_files(self):
        # Package.files
        self.assertHasAttr(self.package, "files")
        self.assertHasType(self.package.files, list)

    def test_group(self):
        # Package.group
        self.assertHasAttr(self.package, "group")
        self.assertHasType(self.package.group, str)

    def test_hdr_chksum(self):
        # Package.hdr_chksum
        self.assertHasAttr(self.package, "hdr_chksum")
        # The value is missing here so the type is type(None) but it should be str
        # self.assertHasType(self.package.hdr_chksum, str)
        self.package.hdr_chksum

    def test_hdr_end(self):
        # Package.hdr_end
        self.assertHasAttr(self.package, "hdr_end")
        self.assertHasType(self.package.hdr_end, int)

    def test_changelogs(self):
        # Package.changelogs
        self.assertHasAttr(self.package, "changelogs")
        self.assertHasType(self.package.changelogs, list)

    def test_installed(self):
        # Package.installed
        self.assertHasAttr(self.package, "installed")
        self.assertHasType(self.package.installed, bool)

    def test_installtime(self):
        # Package.installtime
        self.assertHasAttr(self.package, "installtime")
        self.assertHasType(self.package.installtime, int)

    def test_installsize(self):
        # Package.installsize
        self.assertHasAttr(self.package, "installsize")
        self.assertHasType(self.package.installsize, int)

    def test_license(self):
        # Package.license
        self.assertHasAttr(self.package, "license")
        self.assertHasType(self.package.license, str)

    def test_medianr(self):
        # Package.medianr
        self.assertHasAttr(self.package, "medianr")
        self.assertHasType(self.package.medianr, int)

    def test_name(self):
        # Package.name
        self.assertHasAttr(self.package, "name")
        self.assertHasType(self.package.name, str)

    def test_vendor(self):
        # Package.vendor
        self.assertHasAttr(self.package, "vendor")
        self.assertHasType(self.package.vendor, str)

    def test_obsoletes(self):
        # Package.obsoletes
        self.assertHasAttr(self.package, "obsoletes")
        self.assertHasType(self.package.obsoletes, list)

    def test_provides(self):
        # Package.provides
        self.assertHasAttr(self.package, "provides")
        self.assertHasType(self.package.provides, list)

    def test_recommends(self):
        # Package.recommends
        self.assertHasAttr(self.package, "recommends")
        self.assertHasType(self.package.recommends, list)

    def test_release(self):
        # Package.release
        self.assertHasAttr(self.package, "release")
        self.assertHasType(self.package.release, str)

    def test_reponame(self):
        # Package.reponame
        self.assertHasAttr(self.package, "reponame")
        self.assertHasType(self.package.reponame, str)

    def test_requires(self):
        # Package.requires
        self.assertHasAttr(self.package, "requires")
        self.assertHasType(self.package.requires, list)

    def test_requires_pre(self):
        # Package.requires_pre
        self.assertHasAttr(self.package, "requires_pre")
        self.assertHasType(self.package.requires_pre, list)

    def test_regular_requires(self):
        # Package.regular_requires
        self.assertHasAttr(self.package, "regular_requires")
        self.assertHasType(self.package.regular_requires, list)

    def test_prereq_ignoreinst(self):
        # Package.prereq_ignoreinst
        self.assertHasAttr(self.package, "prereq_ignoreinst")
        self.assertHasType(self.package.prereq_ignoreinst, list)

    def test_rpmdbid(self):
        # Package.rpmdbid
        self.assertHasAttr(self.package, "rpmdbid")
        self.assertHasType(self.package.rpmdbid, int)

    def test_source_debug_name(self):
        # Package.source_debug_name
        self.assertHasAttr(self.package, "source_debug_name")
        self.assertHasType(self.package.source_debug_name, str)

    def test_source_name(self):
        # Package.source_name
        self.assertHasAttr(self.package, "source_name")
        self.assertHasType(self.package.source_name, str)

    def test_debugsource_name(self):
        # Package.debugsource_name
        self.assertHasAttr(self.package, "debugsource_name")
        self.assertHasType(self.package.debugsource_name, str)

    def test_sourcerpm(self):
        # Package.sourcerpm
        self.assertHasAttr(self.package, "sourcerpm")
        self.assertHasType(self.package.sourcerpm, str)

    def test_suggests(self):
        # Package.suggests
        self.assertHasAttr(self.package, "suggests")
        self.assertHasType(self.package.suggests, list)

    def test_summary(self):
        # Package.summary
        self.assertHasAttr(self.package, "summary")
        self.assertHasType(self.package.summary, str)

    def test_supplements(self):
        # Package.supplements
        self.assertHasAttr(self.package, "supplements")
        self.assertHasType(self.package.supplements, list)

    def test_url(self):
        # Package.url
        self.assertHasAttr(self.package, "url")
        # The value is missing here so the type is type(None) but it should be str
        # self.assertHasType(self.package.url, str)
        self.package.url

    def test_version(self):
        # Package.version
        self.assertHasAttr(self.package, "version")
        self.assertHasType(self.package.version, str)

    def test_packager(self):
        # Package.packager
        self.assertHasAttr(self.package, "packager")
        # The value is missing here so the type is type(None) but it should be str
        # self.assertHasType(self.package.packager, str)
        self.package.packager

    def test_remote_location(self):
        # Package.remote_location
        self.assertHasAttr(self.package, "remote_location")
        # This fails due to a bug (filed RhBug:1873146)
        #self.package.remote_location(schemes='http')

    def test_debuginfo_suffix(self):
        # Package.DEBUGINFO_SUFFIX
        self.assertHasAttr(self.package, "DEBUGINFO_SUFFIX")
        self.assertHasType(self.package.DEBUGINFO_SUFFIX, str)

    def test_debugsource_suffix(self):
        # Package.DEBUGSOURCE_SUFFIX
        self.assertHasAttr(self.package, "DEBUGSOURCE_SUFFIX")
        self.assertHasType(self.package.DEBUGSOURCE_SUFFIX, str)




# -*- coding: utf-8 -*-

# Copyright (C) 2014-2018 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#


from __future__ import absolute_import
from __future__ import unicode_literals

import io

import dnf.cli.term

import tests.support
from tests.support import mock


class TermTest(tests.support.TestCase):

    """Tests of ```dnf.cli.term.Term``` class."""

    def test_mode_tty(self):
        """Test whether all modes are properly set if the stream is a tty.

        It also ensures that all the values are unicode strings.

        """
        tty = mock.create_autospec(io.IOBase)
        tty.isatty.return_value = True

        def tigetstr(name):
            return '<cap_%(name)s>' % locals()

        with mock.patch('curses.tigetstr', autospec=True, side_effect=tigetstr):
            term = dnf.cli.term.Term(tty)

        self.assertEqual(term.MODE,
                         {u'blink': tigetstr(u'blink'),
                          u'bold': tigetstr(u'bold'),
                          u'dim': tigetstr(u'dim'),
                          u'normal': tigetstr(u'sgr0'),
                          u'reverse': tigetstr(u'rev'),
                          u'underline': tigetstr(u'smul')})

    def test_mode_tty_incapable(self):
        """Test whether modes correct if the stream is an incapable tty.

        It also ensures that all the values are unicode strings.

        """
        tty = mock.create_autospec(io.IOBase)
        tty.isatty.return_value = True

        with mock.patch('curses.tigetstr', autospec=True, return_value=None):
            term = dnf.cli.term.Term(tty)

        self.assertEqual(term.MODE,
                         {u'blink': u'',
                          u'bold': u'',
                          u'dim': u'',
                          u'normal': u'',
                          u'reverse': u'',
                          u'underline': u''})

    def test_mode_nontty(self):
        """Test whether all modes are properly set if the stream is not a tty.

        It also ensures that all the values are unicode strings.

        """
        nontty = mock.create_autospec(io.IOBase)
        nontty.isatty.return_value = False

        term = dnf.cli.term.Term(nontty)

        self.assertEqual(term.MODE,
                         {u'blink': u'',
                          u'bold': u'',
                          u'dim': u'',
                          u'normal': u'',
                          u'reverse': u'',
                          u'underline': u''})



# -*- coding: utf-8 -*-

# Copyright (C) 2015-2018 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#

from __future__ import absolute_import
from __future__ import unicode_literals

import dnf

import tests.support


class DowngradeTo(tests.support.ResultTestCase):

    REPOS = ['main', 'old_versions']

    def test_downgrade_to_lowest(self):
        with tests.support.mock.patch('logging.Logger.warning'):
            with self.assertRaises(dnf.exceptions.PackagesNotInstalledError):
                self.base.downgrade_to('hole')
        self.assertResult(self.base, self.base._sack.query().installed())

    def test_downgrade_to_name(self):
        self.base.downgrade_to('tour')
        (installed, removed) = self.installed_removed(self.base)
        self.assertCountEqual(map(str, installed),
                              ('tour-4.9-1.noarch',))
        self.assertCountEqual(map(str, removed),
                              ('tour-5-0.noarch',))

    def test_downgrade_to_wildcard_name(self):
        self.base.downgrade_to('tour*')
        (installed, removed) = self.installed_removed(self.base)
        self.assertCountEqual(map(str, installed),
                              ('tour-4.9-1.noarch',))
        self.assertCountEqual(map(str, removed),
                              ('tour-5-0.noarch',))

    def test_downgrade_to_version(self):
        self.base.downgrade_to('tour-4.6')
        (installed, removed) = self.installed_removed(self.base)
        self.assertCountEqual(map(str, installed),
                              ('tour-4.6-1.noarch',))
        self.assertCountEqual(map(str, removed),
                              ('tour-5-0.noarch',))



#!/usr/bin/python3

# script to get rid of
# "'msgid' and 'msgstr' entries do not both begin/end with '\n'"
# error messages during .po to .mo files conversion via msgfmt tool
#
# 'polib' module is needed for run: https://pypi.python.org/pypi/polib
#
# usage: python3 sanitize_po_files.py [po_file...]
#
# in order to update translations from zanata, do:
#  * cmake .
#  * make gettext-update
#  * git add po/*.po
#  * ./scripts/sanitize_po_files.py po/*.po
#  * git commit -m "zanata update"

import polib
import re
import sys


def sanitize_po_file(po_file):
    print("Processing", po_file)
    po = polib.pofile(po_file)
    for entry in po:
        msgid_without_indents = entry.msgid.strip()
        msgstr_without_indents = entry.msgstr.strip()
        entry.msgstr = entry.msgid.replace(
            msgid_without_indents, msgstr_without_indents)
        if re.match(r"^\s+$", entry.msgstr):
            entry.msgstr = ""

        if entry.msgid_plural:
            msgid_plural_without_indents = entry.msgid_plural.strip()
            for i in entry.msgstr_plural.keys():
                msgstr_plural_without_indents = entry.msgstr_plural[i].strip()
                entry.msgstr_plural[i] = entry.msgid_plural.replace(
                    msgid_plural_without_indents,
                    msgstr_plural_without_indents)
                if re.match(r"^\s+$", entry.msgstr_plural[i]):
                    entry.msgstr_plural[i] = ""
    po.save()


if __name__ == "__main__":
    for po_file in sys.argv[1:]:
        sanitize_po_file(po_file)



# -*- coding: utf-8 -*-


from __future__ import absolute_import
from __future__ import unicode_literals

import dnf

from .common import TestCase


class DnfTransactionApiTest(TestCase):
    def test_pkg_action_constants(self):
        self.assertHasAttr(dnf.transaction, "PKG_DOWNGRADE")
        self.assertHasType(dnf.transaction.PKG_DOWNGRADE, int)

        self.assertHasAttr(dnf.transaction, "PKG_DOWNGRADED")
        self.assertHasType(dnf.transaction.PKG_DOWNGRADED, int)

        self.assertHasAttr(dnf.transaction, "PKG_INSTALL")
        self.assertHasType(dnf.transaction.PKG_INSTALL, int)

        self.assertHasAttr(dnf.transaction, "PKG_OBSOLETE")
        self.assertHasType(dnf.transaction.PKG_OBSOLETE, int)

        self.assertHasAttr(dnf.transaction, "PKG_OBSOLETED")
        self.assertHasType(dnf.transaction.PKG_OBSOLETED, int)

        self.assertHasAttr(dnf.transaction, "PKG_REINSTALL")
        self.assertHasType(dnf.transaction.PKG_REINSTALL, int)

        self.assertHasAttr(dnf.transaction, "PKG_REINSTALLED")
        self.assertHasType(dnf.transaction.PKG_REINSTALLED, int)

        self.assertHasAttr(dnf.transaction, "PKG_REMOVE")
        self.assertHasType(dnf.transaction.PKG_REMOVE, int)

        self.assertHasAttr(dnf.transaction, "PKG_UPGRADE")
        self.assertHasType(dnf.transaction.PKG_UPGRADE, int)

        self.assertHasAttr(dnf.transaction, "PKG_UPGRADED")
        self.assertHasType(dnf.transaction.PKG_UPGRADED, int)

        self.assertHasAttr(dnf.transaction, "PKG_ERASE")
        self.assertHasType(dnf.transaction.PKG_ERASE, int)

        self.assertHasAttr(dnf.transaction, "PKG_CLEANUP")
        self.assertHasType(dnf.transaction.PKG_CLEANUP, int)

        self.assertHasAttr(dnf.transaction, "PKG_VERIFY")
        self.assertHasType(dnf.transaction.PKG_VERIFY, int)

        self.assertHasAttr(dnf.transaction, "PKG_SCRIPTLET")
        self.assertHasType(dnf.transaction.PKG_SCRIPTLET, int)

    def test_trans_action_constants(self):
        self.assertHasAttr(dnf.transaction, "TRANS_PREPARATION")
        self.assertHasType(dnf.transaction.TRANS_PREPARATION, int)

        self.assertHasAttr(dnf.transaction, "TRANS_POST")
        self.assertHasType(dnf.transaction.TRANS_POST, int)

    def test_forward_action_constants(self):
        self.assertHasAttr(dnf.transaction, "FORWARD_ACTIONS")
        self.assertHasType(dnf.transaction.FORWARD_ACTIONS, list)

    def test_backward_action_constants(self):
        self.assertHasAttr(dnf.transaction, "BACKWARD_ACTIONS")
        self.assertHasType(dnf.transaction.BACKWARD_ACTIONS, list)

    def test_action_constants(self):
        self.assertHasAttr(dnf.transaction, "ACTIONS")
        self.assertHasType(dnf.transaction.ACTIONS, dict)

    def test_file_action_constants(self):
        self.assertHasAttr(dnf.transaction, "FILE_ACTIONS")
        self.assertHasType(dnf.transaction.FILE_ACTIONS, dict)



# -*- coding: utf-8 -*-

# Copyright (C) 2014-2018  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#

import dnf.dnssec

import tests.support


RPM_USER = 'RPM Packager (The guy who creates packages) <packager@example.com>'
EMAIL = 'packager@example.com'
RPM_RAW_KEY = b'-----BEGIN PGP PUBLIC KEY BLOCK-----\n\nmQENBFqWzewBCADMHY8wL2Gm+aeShboOf' \
              b'G96/h/OJ8FHj+xrihPGY34FwLVqtvO+\nlRIO6t3w5y7O1MWzR1ePtQg1T1sg1s9UeoDRMZoV' \
              b'gzZdRXtQH1Jy9xpTeMfL31ml\nc3wOnitcH0AdWhT7xnnlaCeJuMrX82PWtiJNd+Cy+bgyLfL' \
              b'MwX18cimwYCnyIHQ5\n4qiiywXa1paVHkbEJdbd8f4AxArsHX33xCv76wct2uXiHYAHu67rE5' \
              b'FhQ0mEqfIZ\nHwuFljGPj5UMgyd6MRMuL2Ho1hL7FDPIxDBv5tMWeZJszixIsCkA9Kc1ibylD' \
              b'z9I\nx/keH1K9XXSb/o2EIfF6sWYcj0wNvJllIHAfABEBAAG0QlJQTSBQYWNrYWdlciAo\nVG' \
              b'hlIGd1eSB3aG8gY3JlYXRlcyBwYWNrYWdlcykgPHBhY2thZ2VyQGV4YW1wbGUu\nY29tPokBT' \
              b'gQTAQgAOBYhBMwU+9636QKkbYsjdMKSn1VZ4I5DBQJals3sAhsDBQsJ\nCAcCBhUKCQgLAgQW' \
              b'AgMBAh4BAheAAAoJEMKSn1VZ4I5D6icH/2GY2MMnmcDO+ApI\nF6gGg9itzAHLTNcxEMl9ht1' \
              b'Gu3JQ/VdsvMQ6lG+U8yffEmsfQebtt1UaMVXzyt0t\ncSOBJvVEI6qc6skvfea3hqL4srtTOC' \
              b'wu7ZBA9gOE+eB5BWBspFUVIwUO1GKT48uz\nV6CaODLitaxc7NrJ9HYw4C4+ICCfbIJhjM/Hq' \
              b'xLKOqVy/jsySqIHK96RrjY/m8Q9\nBzsi/6fj/GHkewaM8zaeNLHE7MgXFZAuJ8lzjD0r2oBD' \
              b'JGeBKfjjz9xh26YWjG/A\noiolXfliTEox6p0bf820eRhdX9UdSesBROL3rkB+hM5FOT8crSP' \
              b'JsuGZWJ9brikf\nsurWyU+5AQ0EWpbN7AEIAK0/PThuvsLdDGe5HeZn3VdrFcD9QSOV9Xjos8' \
              b'zWkwx8\nv0t4KXPM4GshyU2ddNjgA+00LhzjACm2ropT5vdvKPtf8lRGOcWWHkiMPdDW/R3/\n' \
              b'I5S0Oh1WFNrQZwQSXn/DoPnhUZNGErpZlUzF00BEltwoVWgT1n81Bp486U3oziZn\nUM8kxXXY' \
              b'2PN8aTfWjsxOSmjyu2m7abeyjTqX9s++vUCQhmCNboSY1AhXF8GQl1Ce\nmpHVv0hOuC6Bead' \
              b'ZRXEfEF0sQogswXpFhYG4GFUvVzKBn00/5phNWHvff1GDjzZz\nmVcRPje+rH6gGP8tpQn7NL' \
              b'1SvrazSqSOih//FfV73EMAEQEAAYkBNgQYAQgAIBYh\nBMwU+9636QKkbYsjdMKSn1VZ4I5DB' \
              b'QJals3sAhsMAAoJEMKSn1VZ4I5DshgIALSn\nLy7KL0bcqxoiEZT+P9O+gX34J2NEfITlu3MZ' \
              b'yN26LbyJS7HuN1vmuUc/UM0ff7AW\n6eElCNFr5HQOrFELUZWiX7f4U8ihRN39g1PKRGANlUc' \
              b'Lm1Z/JnKyYbyzRK70o5A3\n5EiXEHwY62c/b8I1N4kzNKpa0eQcJ7F8XoZhau0UsxqueVPeEI' \
              b'aHX+fjbalz67ea\niFTu8MurdGKntVE1dOPYbGvZE0+HxfDOoVo05bRUH8By7dDgVaI9EijrV' \
              b'zjA5jHP\nJxNIj9AQP9W8zst3l3d9v8o0Pw+L4cWzv+aBFakfjKGugs6lA53fB3RCfZ7OrMwC' \
              b'\nBvlTfDigK9X50K6XoP0=\n=6SRV\n-----END PGP PUBLIC KEY BLOCK-----\n'
ASCII_RAW_KEY = b'mQENBFqWzewBCADMHY8wL2Gm+aeShboOfG96/h/OJ8FHj+xrihPGY34FwLVqtvO+lRIO6t3' \
                b'w5y7O1MWzR1ePtQg1T1sg1s9UeoDRMZoVgzZdRXtQH1Jy9xpTeMfL31mlc3wOnitcH0AdWh' \
                b'T7xnnlaCeJuMrX82PWtiJNd+Cy+bgyLfLMwX18cimwYCnyIHQ54qiiywXa1paVHkbEJdbd8' \
                b'f4AxArsHX33xCv76wct2uXiHYAHu67rE5FhQ0mEqfIZHwuFljGPj5UMgyd6MRMuL2Ho1hL7' \
                b'FDPIxDBv5tMWeZJszixIsCkA9Kc1ibylDz9Ix/keH1K9XXSb/o2EIfF6sWYcj0wNvJllIHA' \
                b'fABEBAAG0QlJQTSBQYWNrYWdlciAoVGhlIGd1eSB3aG8gY3JlYXRlcyBwYWNrYWdlcykgPH' \
                b'BhY2thZ2VyQGV4YW1wbGUuY29tPokBTgQTAQgAOBYhBMwU+9636QKkbYsjdMKSn1VZ4I5DB' \
                b'QJals3sAhsDBQsJCAcCBhUKCQgLAgQWAgMBAh4BAheAAAoJEMKSn1VZ4I5D6icH/2GY2MMn' \
                b'mcDO+ApIF6gGg9itzAHLTNcxEMl9ht1Gu3JQ/VdsvMQ6lG+U8yffEmsfQebtt1UaMVXzyt0' \
                b'tcSOBJvVEI6qc6skvfea3hqL4srtTOCwu7ZBA9gOE+eB5BWBspFUVIwUO1GKT48uzV6CaOD' \
                b'Litaxc7NrJ9HYw4C4+ICCfbIJhjM/HqxLKOqVy/jsySqIHK96RrjY/m8Q9Bzsi/6fj/GHke' \
                b'waM8zaeNLHE7MgXFZAuJ8lzjD0r2oBDJGeBKfjjz9xh26YWjG/AoiolXfliTEox6p0bf820' \
                b'eRhdX9UdSesBROL3rkB+hM5FOT8crSPJsuGZWJ9brikfsurWyU+5AQ0EWpbN7AEIAK0/PTh' \
                b'uvsLdDGe5HeZn3VdrFcD9QSOV9Xjos8zWkwx8v0t4KXPM4GshyU2ddNjgA+00LhzjACm2ro' \
                b'pT5vdvKPtf8lRGOcWWHkiMPdDW/R3/I5S0Oh1WFNrQZwQSXn/DoPnhUZNGErpZlUzF00BEl' \
                b'twoVWgT1n81Bp486U3oziZnUM8kxXXY2PN8aTfWjsxOSmjyu2m7abeyjTqX9s++vUCQhmCN' \
                b'boSY1AhXF8GQl1CempHVv0hOuC6BeadZRXEfEF0sQogswXpFhYG4GFUvVzKBn00/5phNWHv' \
                b'ff1GDjzZzmVcRPje+rH6gGP8tpQn7NL1SvrazSqSOih//FfV73EMAEQEAAYkBNgQYAQgAIB' \
                b'YhBMwU+9636QKkbYsjdMKSn1VZ4I5DBQJals3sAhsMAAoJEMKSn1VZ4I5DshgIALSnLy7KL' \
                b'0bcqxoiEZT+P9O+gX34J2NEfITlu3MZyN26LbyJS7HuN1vmuUc/UM0ff7AW6eElCNFr5HQO' \
                b'rFELUZWiX7f4U8ihRN39g1PKRGANlUcLm1Z/JnKyYbyzRK70o5A35EiXEHwY62c/b8I1N4k' \
                b'zNKpa0eQcJ7F8XoZhau0UsxqueVPeEIaHX+fjbalz67eaiFTu8MurdGKntVE1dOPYbGvZE0' \
                b'+HxfDOoVo05bRUH8By7dDgVaI9EijrVzjA5jHPJxNIj9AQP9W8zst3l3d9v8o0Pw+L4cWzv' \
                b'+aBFakfjKGugs6lA53fB3RCfZ7OrMwCBvlTfDigK9X50K6XoP0='


class EmailToLocationTest(tests.support.TestCase):

    def test_convert_simple_email_to_domain(self):
        input = 'hugh@example.com'
        output = 'c93f1e400f26708f98cb19d936620da35eec8f72e57f9eec01c1afd6._openpgpkey.example.com'
        self.assertEqual(dnf.dnssec.email2location(input), output)


class KeyInfoTest(tests.support.TestCase):

    def test_key_info_from_rpm_key_object_email_part(self):
        key_info = dnf.dnssec.KeyInfo.from_rpm_key_object(RPM_USER, RPM_RAW_KEY)
        self.assertEqual(key_info.email, EMAIL)

    def test_key_info_from_rpm_key_object_key_part(self):
        key_info = dnf.dnssec.KeyInfo.from_rpm_key_object(RPM_USER, RPM_RAW_KEY)
        self.assertEqual(key_info.key, ASCII_RAW_KEY)



# -*- coding: utf-8 -*-

# Copyright (C) 2012-2018 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#

from __future__ import absolute_import
from __future__ import unicode_literals

import libdnf.transaction

import dnf.history

import tests.support
from tests.support import mock

'''
class NEVRAOperationsTest(tests.support.TestCase):
    """Unit tests of dnf.history.NEVRAOperations."""

    def test_add_erase_installed(self):
        """Test add with an erasure of NEVRA which was installed before."""
        ops = dnf.history.NEVRAOperations()
        ops.add('Install', 'tour-0:4.6-1.noarch', obsoleted_nevras=('lotus-0:3-16.x86_64',))
        ops.add('Erase', 'tour-0:4.6-1.noarch')

        self.assertCountEqual(
            ops,
            (('Erase', 'lotus-0:3-16.x86_64', None, set()),))

    def test_add_erase_removed(self):
        """Test add with an erasure of NEVRA which was removed before."""
        ops = dnf.history.NEVRAOperations()
        ops.add('Erase', 'tour-0:4.6-1.noarch')

        self.assertRaises(
            ValueError,
            ops.add, 'Erase', 'tour-0:4.6-1.noarch')

    def test_add_install_installed(self):
        """Test add with two installs of the same NEVRA."""
        ops = dnf.history.NEVRAOperations()
        ops.add('Install', 'tour-0:4.6-1.noarch')

        self.assertRaises(
            ValueError,
            ops.add, 'Install', 'tour-0:4.6-1.noarch')

    def test_add_install_removed(self):
        """Test add with an install of NEVRA which was removed before."""
        ops = dnf.history.NEVRAOperations()
        ops.add('Erase', 'tour-0:4.6-1.noarch')
        ops.add('Install', 'tour-0:4.6-1.noarch')

        self.assertCountEqual(
            ops,
            (('Reinstall', 'tour-0:4.6-1.noarch', 'tour-0:4.6-1.noarch', set()),))

    def test_add_obsoleted_installed(self):
        """Test add with an obsoleted NEVRA which was installed before."""
        ops = dnf.history.NEVRAOperations()
        ops.add('Install', 'lotus-0:3-16.x86_64')
        ops.add('Install', 'tour-0:4.6-1.noarch', obsoleted_nevras=('lotus-0:3-16.x86_64',))

        self.assertCountEqual(
            ops,
            (('Install', 'tour-0:4.6-1.noarch', None, set()),))

    def test_add_obsoleted_obsoleted(self):
        """Test add with an obsoleted NEVRA which was obsoleted before."""
        ops = dnf.history.NEVRAOperations()
        ops.add(
            'Install',
            'tour-0:4.6-1.noarch',
            obsoleted_nevras=('lotus-0:3-16.x86_64', 'mrkite-0:2-0.x86_64')
        )
        ops.add(
            'Install',
            'pepper-0:20-0.x86_64',
            obsoleted_nevras=('lotus-0:3-16.x86_64', 'librita-0:1-1.x86_64')
        )

        self.assertCountEqual(
            ops,
            (
                (
                    'Install',
                    'tour-0:4.6-1.noarch',
                    None,
                    {'lotus-0:3-16.x86_64', 'mrkite-0:2-0.x86_64'}
                ),
                (
                    'Install',
                    'pepper-0:20-0.x86_64',
                    None,
                    {'lotus-0:3-16.x86_64', 'librita-0:1-1.x86_64'}
                )
            )
        )

    def test_add_obsoleted_removed(self):
        """Test add with an obsoleted NEVRA which was removed before."""
        ops = dnf.history.NEVRAOperations()
        ops.add('Erase', 'lotus-0:3-16.x86_64')

        self.assertRaises(
            ValueError,
            ops.add, 'Install', 'tour-0:4.6-1.noarch', obsoleted_nevras=('lotus-0:3-16.x86_64',))

    def test_add_reinstall_installed(self):
        """Test add with a reinstall of NEVRA which was installed before."""
        ops = dnf.history.NEVRAOperations()
        ops.add('Install', 'tour-0:4.6-1.noarch')
        ops.add('Reinstall', 'tour-0:4.6-1.noarch', 'tour-0:4.6-1.noarch')

        self.assertCountEqual(
            ops,
            (('Install', 'tour-0:4.6-1.noarch', None, set()),))

    def test_add_replace_installed(self):
        """Test add with a replacing NEVRA which was installed before."""
        ops = dnf.history.NEVRAOperations()
        ops.add('Install', 'tour-0:4.8-1.noarch')

        self.assertRaises(
            ValueError,
            ops.add, 'Update', 'tour-0:4.8-1.noarch', 'tour-0:4.6-1.noarch')

    def test_add_replace_opposite(self):
        """Test add with a replacement which was done before, but swapped."""
        ops = dnf.history.NEVRAOperations()
        ops.add('Downgrade', 'tour-0:4.6-1.noarch', 'tour-0:4.8-1.noarch')
        ops.add('Update', 'tour-0:4.8-1.noarch', 'tour-0:4.6-1.noarch')

        self.assertCountEqual(
            ops,
            (('Reinstall', 'tour-0:4.8-1.noarch', 'tour-0:4.8-1.noarch', set()),))

    def test_add_replace_opposite_manual(self):
        """Test add with a manual replacement which was done before, but swapped."""
        ops = dnf.history.NEVRAOperations()
        ops.add('Erase', 'tour-0:4.8-1.noarch')
        ops.add('Install', 'tour-0:4.6-1.noarch')
        ops.add('Update', 'tour-0:4.8-1.noarch', 'tour-0:4.6-1.noarch')

        self.assertCountEqual(
            ops,
            (('Reinstall', 'tour-0:4.8-1.noarch', 'tour-0:4.8-1.noarch', set()),))

    def test_add_replace_removed(self):
        """Test add with a replacing NEVRA which was removed before."""
        ops = dnf.history.NEVRAOperations()
        ops.add('Erase', 'tour-0:4.8-1.noarch')
        ops.add('Update', 'tour-0:4.8-1.noarch', 'tour-0:4.6-1.noarch')

        self.assertCountEqual(
            ops,
            (('Reinstall', 'tour-0:4.8-1.noarch', 'tour-0:4.8-1.noarch', set()),
             ('Erase', 'tour-0:4.6-1.noarch', None, set())))

    def test_add_replaced_opposite(self):
        """Test add with a replaced NEVRA which replaced a NEVRA before in the opposite direction."""
        ops = dnf.history.NEVRAOperations()
        ops.add('Downgrade', 'tour-0:4.6-1.noarch', 'tour-0:4.9-1.noarch')
        ops.add('Update', 'tour-0:4.8-1.noarch', 'tour-0:4.6-1.noarch')

        self.assertCountEqual(
            ops,
            (('Erase', 'tour-0:4.9-1.noarch', None, set()),
             ('Install', 'tour-0:4.8-1.noarch', None, set())))

    def test_add_replaced_removed(self):
        """Test add with a replaced NEVRA which was removed before."""
        ops = dnf.history.NEVRAOperations()
        ops.add('Erase', 'tour-0:4.6-1.noarch')

        self.assertRaises(
            ValueError,
            ops.add, 'Update', 'tour-0:4.8-1.noarch', 'tour-0:4.6-1.noarch')

    def test_add_replaced_reinstall(self):
        """Test add with a replaced NEVRA which was reinstalled before."""
        ops = dnf.history.NEVRAOperations()
        ops.add('Reinstall', 'tour-0:4.6-1.noarch', 'tour-0:4.6-1.noarch')
        ops.add('Update', 'tour-0:4.8-1.noarch', 'tour-0:4.6-1.noarch')

        self.assertCountEqual(
            ops,
            (('Update', 'tour-0:4.8-1.noarch', 'tour-0:4.6-1.noarch', set()),))

    def test_add_replaced_replacement(self):
        """Test add with a replaced NEVRA which replaced a NEVRA before."""
        ops = dnf.history.NEVRAOperations()
        ops.add('Update', 'tour-0:4.8-1.noarch', 'tour-0:4.6-1.noarch')
        ops.add('Update', 'tour-0:4.9-1.noarch', 'tour-0:4.8-1.noarch')

        self.assertCountEqual(
            ops,
            (('Update', 'tour-0:4.9-1.noarch', 'tour-0:4.6-1.noarch', set()),))

    def test_addition(self):
        """Test addition of two instances."""
        left_ops = dnf.history.NEVRAOperations()
        left_ops.add('Update', 'tour-0:4.8-1.noarch', 'tour-0:4.6-1.noarch')
        right_ops = dnf.history.NEVRAOperations()
        right_ops.add('Update', 'tour-0:4.9-1.noarch', 'tour-0:4.8-1.noarch')
        expected_ops = dnf.history.NEVRAOperations()
        expected_ops.add('Update', 'tour-0:4.9-1.noarch', 'tour-0:4.6-1.noarch')

        result_ops = left_ops + right_ops

        self.assertEqual(result_ops, expected_ops)

    def test_addition_inplace(self):
        """Test in-place addition of two instances."""
        left_ops = dnf.history.NEVRAOperations()
        left_ops.add('Update', 'tour-0:4.8-1.noarch', 'tour-0:4.6-1.noarch')
        right_ops = dnf.history.NEVRAOperations()
        right_ops.add('Update', 'tour-0:4.9-1.noarch', 'tour-0:4.8-1.noarch')
        expected_ops = dnf.history.NEVRAOperations()
        expected_ops.add('Update', 'tour-0:4.9-1.noarch', 'tour-0:4.6-1.noarch')

        left_ops += right_ops

        self.assertEqual(left_ops, expected_ops)

    def test_equality(self):
        """Test equality of two equal instances."""
        left_ops = dnf.history.NEVRAOperations()
        left_ops.add('Update', 'tour-0:4.8-1.noarch', 'tour-0:4.6-1.noarch')
        right_ops = dnf.history.NEVRAOperations()
        right_ops.add('Update', 'tour-0:4.8-1.noarch', 'tour-0:4.6-1.noarch')

        is_equal = left_ops == right_ops

        self.assertTrue(is_equal)

    def test_equality_differentcontent(self):
        """Test equality of two instances with different contents."""
        left_ops = dnf.history.NEVRAOperations()
        left_ops.add('Downgrade', 'tour-0:4.6-1.noarch', 'tour-0:4.8-1.noarch')
        right_ops = dnf.history.NEVRAOperations()
        right_ops.add('Update', 'tour-0:4.8-1.noarch', 'tour-0:4.6-1.noarch')

        is_equal = left_ops == right_ops

        self.assertFalse(is_equal)

    def test_equality_differentlength(self):
        """Test equality of two instances with different lengths."""
        left_ops = dnf.history.NEVRAOperations()
        right_ops = dnf.history.NEVRAOperations()
        right_ops.add('Update', 'tour-0:4.8-1.noarch', 'tour-0:4.6-1.noarch')

        is_equal = left_ops == right_ops

        self.assertFalse(is_equal)

    def test_equality_differenttype(self):
        """Test equality of an instance and an object of a different type."""
        ops = dnf.history.NEVRAOperations()
        ops.add('Update', 'tour-0:4.8-1.noarch', 'tour-0:4.6-1.noarch')

        is_equal = ops == 'tour-0:4.8-1.noarch'

        self.assertFalse(is_equal)

    def test_equality_identity(self):
        """Test equality of the same instance."""
        ops = dnf.history.NEVRAOperations()
        ops.add('Update', 'tour-0:4.8-1.noarch', 'tour-0:4.6-1.noarch')

        is_equal = ops == ops

        self.assertTrue(is_equal)

    def test_inequality(self):
        """Test inequality of two different instances."""
        left_ops = dnf.history.NEVRAOperations()
        left_ops.add('Downgrade', 'tour-0:4.6-1.noarch', 'tour-0:4.8-1.noarch')
        right_ops = dnf.history.NEVRAOperations()
        right_ops.add('Update', 'tour-0:4.8-1.noarch', 'tour-0:4.6-1.noarch')

        is_inequal = left_ops != right_ops

        self.assertTrue(is_inequal)

    def test_inequality_equal(self):
        """Test inequality of two equal instances."""
        left_ops = dnf.history.NEVRAOperations()
        left_ops.add('Update', 'tour-0:4.8-1.noarch', 'tour-0:4.6-1.noarch')
        right_ops = dnf.history.NEVRAOperations()
        right_ops.add('Update', 'tour-0:4.8-1.noarch', 'tour-0:4.6-1.noarch')

        is_inequal = left_ops != right_ops

        self.assertFalse(is_inequal)

    def test_iterator(self):
        """Test iterator of an instance."""
        ops = dnf.history.NEVRAOperations()
        ops.add('Update', 'tour-0:4.8-1.noarch', 'tour-0:4.6-1.noarch')

        iterator = iter(ops)

        self.assertEqual(
            next(iterator),
            ('Update', 'tour-0:4.8-1.noarch', 'tour-0:4.6-1.noarch', set()))
        self.assertRaises(StopIteration, next, iterator)

    def test_length(self):
        """Test length of an instance."""
        ops = dnf.history.NEVRAOperations()
        ops.add('Update', 'tour-0:4.8-1.noarch', 'tour-0:4.6-1.noarch')

        length = len(ops)

        self.assertEqual(length, 1)

    def test_membership(self):
        """Test membership of a contained operation."""
        ops = dnf.history.NEVRAOperations()
        ops.add('Update', 'tour-0:4.9-1.noarch', 'tour-0:4.8-1.noarch')

        is_in = ('Update', 'tour-0:4.9-1.noarch', 'tour-0:4.8-1.noarch', ()) in ops

        self.assertTrue(is_in)

    def test_membership_differentnevra(self):
        """Test membership of an operation with different (replacing) NEVRA."""
        ops = dnf.history.NEVRAOperations()
        ops.add('Update', 'tour-0:4.9-1.noarch', 'tour-0:4.8-1.noarch')

        is_in = ('Update', 'pepper-0:20-0.x86_64', 'tour-0:4.8-1.noarch', ()) in ops

        self.assertFalse(is_in)

    def test_membership_differentobsoleted(self):
        """Test membership of an operation with different obsoleted NEVRAs."""
        ops = dnf.history.NEVRAOperations()
        ops.add('Update', 'tour-0:4.9-1.noarch', 'tour-0:4.8-1.noarch')
        op = (
            'Update',
            'tour-0:4.9-1.noarch',
            'tour-0:4.8-1.noarch',
            ('pepper-0:20-0.x86_64',)
        )
        self.assertFalse(op in ops)

    def test_membership_differentreplaced(self):
        """Test membership of an operation with different replaced NEVRA."""
        ops = dnf.history.NEVRAOperations()
        ops.add('Update', 'tour-0:4.9-1.noarch', 'tour-0:4.8-1.noarch')

        is_in = ('Update', 'tour-0:4.9-1.noarch', 'pepper-0:20-0.x86_64', ()) in ops

        self.assertFalse(is_in)

    def test_membership_differentstate(self):
        """Test membership of an operation with different state."""
        ops = dnf.history.NEVRAOperations()
        ops.add('Update', 'tour-0:4.9-1.noarch', 'tour-0:4.8-1.noarch')

        is_in = ('Downgrade', 'tour-0:4.9-1.noarch', 'tour-0:4.8-1.noarch', ()) in ops

        self.assertFalse(is_in)

    def test_membership_differenttype(self):
        """Test membership of an object of a different type."""
        ops = dnf.history.NEVRAOperations()
        ops.add('Update', 'tour-0:4.9-1.noarch', 'tour-0:4.8-1.noarch')

        is_in = 'tour-0:4.9-1.noarch' in ops

        self.assertFalse(is_in)


class TransactionConverterTest(tests.support.TestCase):
    """Unit tests of dnf.history.TransactionConverter."""

    def assert_transaction_equal(self, actual, expected):
        """Assert that two transactions are equal."""
        self.assertCountEqual(self.transaction2tuples(actual),
                              self.transaction2tuples(expected))

    def test_find_available_na(self):
        """Test finding with an unavailable NEVRA."""
        sack = tests.support.mock_sack('main')
        converter = dnf.history.TransactionConverter(sack)
        with self.assertRaises(dnf.exceptions.PackagesNotAvailableError) as ctx:
            converter._find_available('none-1-0.noarch')

        self.assertEqual(ctx.exception.pkg_spec, 'none-1-0.noarch')

    def test_find_installed_ni(self):
        """Test finding with an unistalled NEVRA."""
        sack = tests.support.mock_sack('main')
        converter = dnf.history.TransactionConverter(sack)
        with self.assertRaises(dnf.exceptions.PackagesNotInstalledError) as ctx:
            converter._find_installed('none-1-0.noarch')

        self.assertEqual(ctx.exception.pkg_spec, 'none-1-0.noarch')

    def test_convert_downgrade(self):
        """Test conversion with a downgrade."""
        operations = dnf.history.NEVRAOperations()
        operations.add('Downgrade', 'tour-4.6-1.noarch', 'tour-5-0.noarch',
                       ['hole-1-1.x86_64'])

        sack = tests.support.mock_sack('main')
        converter = dnf.history.TransactionConverter(sack)
        actual = converter.convert(operations)

        expected = dnf.transaction.Transaction()
        expected.add_downgrade(
            next(iter(sack.query().available()._nevra('tour-4.6-1.noarch'))),
            next(iter(sack.query().installed()._nevra('tour-5-0.noarch'))),
            [next(iter(sack.query().installed()._nevra('hole-1-1.x86_64')))])
        self.assert_transaction_equal(actual, expected)

    def test_convert_erase(self):
        """Test conversion with an erasure."""
        operations = dnf.history.NEVRAOperations()
        operations.add('Erase', 'pepper-20-0.x86_64')

        sack = tests.support.mock_sack()
        converter = dnf.history.TransactionConverter(sack)
        actual = converter.convert(operations)

        expected = dnf.transaction.Transaction()
        expected.add_erase(
            next(iter(sack.query().installed()._nevra('pepper-20-0.x86_64'))))
        self.assert_transaction_equal(actual, expected)

    def test_convert_install(self):
        """Test conversion with an installation."""
        operations = dnf.history.NEVRAOperations()
        operations.add('Install', 'lotus-3-16.x86_64',
                       obsoleted_nevras=['hole-1-1.x86_64'])

        sack = tests.support.mock_sack('main')
        converter = dnf.history.TransactionConverter(sack)
        actual = converter.convert(operations, libdnf.transaction.TransactionItemReason_USER)

        expected = dnf.transaction.Transaction()
        expected.add_install(
            next(iter(sack.query().available()._nevra('lotus-3-16.x86_64'))),
            [next(iter(sack.query().installed()._nevra('hole-1-1.x86_64')))],
            libdnf.transaction.TransactionItemReason_USER)
        self.assert_transaction_equal(actual, expected)

    def test_convert_reinstall(self):
        """Test conversion with a reinstallation."""
        operations = dnf.history.NEVRAOperations()
        operations.add('Reinstall', 'pepper-20-0.x86_64', 'pepper-20-0.x86_64',
                       ['hole-1-1.x86_64'])

        sack = tests.support.mock_sack('main')
        converter = dnf.history.TransactionConverter(sack)
        actual = converter.convert(operations)

        expected = dnf.transaction.Transaction()
        expected.add_reinstall(
            next(iter(sack.query().available()._nevra('pepper-20-0.x86_64'))),
            next(iter(sack.query().installed()._nevra('pepper-20-0.x86_64'))),
            [next(iter(sack.query().installed()._nevra('hole-1-1.x86_64')))])
        self.assert_transaction_equal(actual, expected)

    def test_upgrade(self):
        """Test repeating with an upgrade."""
        operations = dnf.history.NEVRAOperations()
        operations.add('Update', 'pepper-20-1.x86_64', 'pepper-20-0.x86_64',
                       ['hole-1-1.x86_64'])

        sack = tests.support.mock_sack('updates')
        converter = dnf.history.TransactionConverter(sack)
        actual = converter.convert(operations)

        expected = dnf.transaction.Transaction()
        expected.add_upgrade(
            next(iter(sack.query().available()._nevra('pepper-20-1.x86_64'))),
            next(iter(sack.query().installed()._nevra('pepper-20-0.x86_64'))),
            [next(iter(sack.query().installed()._nevra('hole-1-1.x86_64')))])
        self.assert_transaction_equal(actual, expected)

    @staticmethod
    def transaction2tuples(transaction):
        """Convert a transaction to the iterable of tuples."""
        for item in transaction:
            yield (item.op_type, item.installed, item.erased, item.obsoleted,
                   item.reason)
'''



# -*- coding: utf-8 -*-


from __future__ import absolute_import
from __future__ import unicode_literals

import dnf
import libcomps

from .common import TestCase


class MockLangs():
    def get(self):
        return []


class DnfCompsApiTest(TestCase):
    def test_conditional(self):
        # dnf.comps.CONDITIONAL
        self.assertHasAttr(dnf.comps, "CONDITIONAL")
        self.assertHasType(dnf.comps.CONDITIONAL, int)

    def test_default(self):
        # dnf.comps.DEFAULT
        self.assertHasAttr(dnf.comps, "DEFAULT")
        self.assertHasType(dnf.comps.DEFAULT, int)

    def test_mandatory(self):
        # dnf.comps.MANDATORY
        self.assertHasAttr(dnf.comps, "MANDATORY")
        self.assertHasType(dnf.comps.MANDATORY, int)

    def test_optional(self):
        # dnf.comps.OPTIONAL
        self.assertHasAttr(dnf.comps, "OPTIONAL")
        self.assertHasType(dnf.comps.OPTIONAL, int)

    def test_category(self):
        # dnf.comps.Category
        self.assertHasAttr(dnf.comps, "Category")
        self.assertHasType(dnf.comps.Category, object)

    def test_category_init(self):
        libcomps_category = libcomps.Category("id", "name", "description")
        _ = dnf.comps.Category(iobj=libcomps_category, langs=None, group_factory=None)

    def test_category_id(self):
        # dnf.comps.Category.id
        libcomps_category = libcomps.Category("id", "name", "description")
        category = dnf.comps.Category(iobj=libcomps_category, langs=None, group_factory=None)

        self.assertHasAttr(category, "id")
        self.assertHasType(category.id, str)

    def test_category_name(self):
        # dnf.comps.Category.name
        libcomps_category = libcomps.Category("id", "name", "description")
        category = dnf.comps.Category(iobj=libcomps_category, langs=None, group_factory=None)

        self.assertHasAttr(category, "name")
        self.assertHasType(category.name, str)

    def test_category_ui_name(self):
        # dnf.comps.Category.ui_name
        libcomps_category = libcomps.Category("id", "name", "description")
        langs = MockLangs()
        category = dnf.comps.Category(iobj=libcomps_category, langs=langs, group_factory=None)

        self.assertHasAttr(category, "ui_name")
        self.assertHasType(category.ui_name, str)

    def test_category_ui_description(self):
        # dnf.comps.Category.ui_description
        libcomps_category = libcomps.Category("id", "name", "description")
        langs = MockLangs()
        category = dnf.comps.Category(iobj=libcomps_category, langs=langs, group_factory=None)

        self.assertHasAttr(category, "ui_description")
        self.assertHasType(category.ui_description, str)

    def test_environment(self):
        # dnf.comps.Environment
        self.assertHasAttr(dnf.comps, "Environment")
        self.assertHasType(dnf.comps.Environment, object)

    def test_environment_init(self):
        libcomps_environment = libcomps.Environment("id", "name", "description")
        _ = dnf.comps.Environment(iobj=libcomps_environment, langs=None, group_factory=None)

    def test_environment_id(self):
        # dnf.comps.Environment.id
        libcomps_environment = libcomps.Environment("id", "name", "description")
        environment = dnf.comps.Environment(iobj=libcomps_environment, langs=None, group_factory=None)

        self.assertHasAttr(environment, "id")
        self.assertHasType(environment.id, str)

    def test_environment_name(self):
        # dnf.comps.Environment.name
        libcomps_environment = libcomps.Environment("id", "name", "description")
        environment = dnf.comps.Environment(iobj=libcomps_environment, langs=None, group_factory=None)

        self.assertHasAttr(environment, "name")
        self.assertHasType(environment.name, str)

    def test_environment_ui_name(self):
        # dnf.comps.Environment.ui_name
        libcomps_environment = libcomps.Environment("id", "name", "description")
        langs = MockLangs()
        environment = dnf.comps.Environment(iobj=libcomps_environment, langs=langs, group_factory=None)

        self.assertHasAttr(environment, "ui_name")
        self.assertHasType(environment.ui_name, str)

    def test_environment_ui_description(self):
        # dnf.comps.Environment.ui_description
        libcomps_environment = libcomps.Environment("id", "name", "description")
        langs = MockLangs()
        environment = dnf.comps.Environment(iobj=libcomps_environment, langs=langs, group_factory=None)

        self.assertHasAttr(environment, "ui_description")
        self.assertHasType(environment.ui_description, str)

    def test_group(self):
        # dnf.comps.Group
        self.assertHasAttr(dnf.comps, "Group")
        self.assertHasType(dnf.comps.Group, object)

    def test_group_init(self):
        libcomps_group = libcomps.Group("id", "name", "description")
        _ = dnf.comps.Group(iobj=libcomps_group, langs=None, pkg_factory=None)

    def test_group_id(self):
        # dnf.comps.Group.id
        libcomps_group = libcomps.Group("id", "name", "description")
        group = dnf.comps.Group(iobj=libcomps_group, langs=None, pkg_factory=None)

        self.assertHasAttr(group, "id")
        self.assertHasType(group.id, str)

    def test_group_name(self):
        # dnf.comps.Group.name
        libcomps_group = libcomps.Group("id", "name", "description")
        group = dnf.comps.Group(iobj=libcomps_group, langs=None, pkg_factory=None)

        self.assertHasAttr(group, "name")
        self.assertHasType(group.name, str)

    def test_group_ui_name(self):
        # dnf.comps.Group.ui_name
        libcomps_group = libcomps.Group("id", "name", "description")
        langs = MockLangs()
        group = dnf.comps.Group(iobj=libcomps_group, langs=langs, pkg_factory=None)

        self.assertHasAttr(group, "ui_name")
        self.assertHasType(group.ui_name, str)

    def test_group_ui_description(self):
        # dnf.comps.Group.ui_description
        libcomps_group = libcomps.Group("id", "name", "description")
        langs = MockLangs()
        group = dnf.comps.Group(iobj=libcomps_group, langs=langs, pkg_factory=None)

        self.assertHasAttr(group, "ui_description")
        self.assertHasType(group.ui_description, str)

    def test_group_packages_iter(self):
        # dnf.comps.Group.packages_iter
        libcomps_group = libcomps.Group("id", "name", "description")
        group = dnf.comps.Group(libcomps_group, None, lambda x: x)
        group.packages_iter()

    def test_package(self):
        # dnf.comps.Package
        self.assertHasAttr(dnf.comps, "Package")
        self.assertHasType(dnf.comps.Package, object)

    def test_package_init(self):
        libcomps_package = libcomps.Package()
        _ = dnf.comps.Package(ipkg=libcomps_package)

    def test_package_name(self):
        # dnf.comps.Package.name
        libcomps_package = libcomps.Package()
        libcomps_package.name = "name"
        package = dnf.comps.Package(libcomps_package)

        self.assertHasAttr(package, "name")
        self.assertHasType(package.name, str)

    def test_package_option_type(self):
        # dnf.comps.Package.option_type
        libcomps_package = libcomps.Package()
        libcomps_package.type = 0
        package = dnf.comps.Package(libcomps_package)

        self.assertHasAttr(package, "option_type")
        self.assertHasType(package.option_type, int)

    def test_comps(self):
        # dnf.comps.Comps
        self.assertHasAttr(dnf.comps, "Comps")
        self.assertHasType(dnf.comps.Comps, object)

    def test_comps_init(self):
        _ = dnf.comps.Comps()

    def test_comps_categories(self):
        # dnf.comps.Comps.categories
        comps = dnf.comps.Comps()

        self.assertHasAttr(comps, "categories")
        self.assertHasType(comps.categories, list)

    def test_comps_category_by_pattern(self):
        # dnf.comps.Comps.category_by_pattern
        comps = dnf.comps.Comps()
        comps.category_by_pattern(pattern="foo", case_sensitive=False)

    def test_comps_categories_by_pattern(self):
        # dnf.comps.Comps.categories_by_pattern
        comps = dnf.comps.Comps()
        comps.categories_by_pattern(pattern="foo", case_sensitive=False)

    def test_comps_categories_iter(self):
        # dnf.comps.Comps.categories_iter
        comps = dnf.comps.Comps()
        comps.categories_iter()

    def test_comps_environments(self):
        # dnf.comps.Comps.environments
        comps = dnf.comps.Comps()

        self.assertHasAttr(comps, "environments")
        self.assertHasType(comps.environments, list)

    def test_comps_environment_by_pattern(self):
        # dnf.comps.Comps.environment_by_pattern
        comps = dnf.comps.Comps()
        comps.environment_by_pattern(pattern="foo", case_sensitive=False)

    def test_comps_environments_by_pattern(self):
        # dnf.comps.Comps.environments_by_pattern
        comps = dnf.comps.Comps()
        comps.environments_by_pattern(pattern="foo", case_sensitive=False)

    def test_comps_environments_iter(self):
        # dnf.comps.Comps.environments_iter
        comps = dnf.comps.Comps()
        comps.environments_iter()

    def test_comps_groups(self):
        # dnf.comps.Comps.groups
        comps = dnf.comps.Comps()

        self.assertHasAttr(comps, "groups")
        self.assertHasType(comps.groups, list)

    def test_comps_group_by_pattern(self):
        # dnf.comps.Comps.group_by_pattern
        comps = dnf.comps.Comps()
        comps.group_by_pattern(pattern="foo", case_sensitive=False)

    def test_comps_groups_by_pattern(self):
        # dnf.comps.Comps.groups_by_pattern
        comps = dnf.comps.Comps()
        comps.groups_by_pattern(pattern="foo", case_sensitive=False)

    def test_comps_groups_iter(self):
        # dnf.comps.Comps.groups_iter
        comps = dnf.comps.Comps()
        comps.groups_iter()



# -*- coding: utf-8 -*-


from __future__ import absolute_import
from __future__ import unicode_literals

import dnf
import dnf.module.module_base

from .common import TestCase


class DnfModuleBaseApiTest(TestCase):
    def setUp(self):
        self.base = dnf.Base(dnf.conf.Conf())
        self.moduleBase = dnf.module.module_base.ModuleBase(self.base)

    def tearDown(self):
        self.base.close()

    def test_init(self):
        moduleBase = dnf.module.module_base.ModuleBase(self.base)

    def test_enable(self):
        # ModuleBase.enable()
        self.assertHasAttr(self.moduleBase, "enable")
        self.assertRaises(
            dnf.exceptions.Error,
            self.moduleBase.enable,
            module_specs=["nodejs:8"],
        )

    def test_disable(self):
        # ModuleBase.disable()
        self.assertHasAttr(self.moduleBase, "disable")
        self.assertRaises(
            dnf.exceptions.Error,
            self.moduleBase.disable,
            module_specs=["nodejs"],
        )

    def test_reset(self):
        # ModuleBase.reset()
        self.assertHasAttr(self.moduleBase, "reset")
        self.assertRaises(
            dnf.exceptions.Error,
            self.moduleBase.reset,
            module_specs=["nodejs:8"],
        )

    def test_install(self):
        # ModuleBase.install()
        self.assertHasAttr(self.moduleBase, "install")
        self.assertRaises(
            AttributeError,
            self.moduleBase.install,
            module_specs=[],
            strict=False,
        )

    def test_remove(self):
        # ModuleBase.remove()
        self.assertHasAttr(self.moduleBase, "remove")
        self.moduleBase.remove(module_specs=[])

    def test_upgrade(self):
        # ModuleBase.upgrade()
        self.assertHasAttr(self.moduleBase, "upgrade")
        self.moduleBase.upgrade(module_specs=[])

    def test_get_modules(self):
        # ModuleBase.get_modules()
        self.assertHasAttr(self.moduleBase, "get_modules")
        self.moduleBase.get_modules(module_spec="")



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
# Copyright 2003 Duke University

from __future__ import print_function, absolute_import
from __future__ import unicode_literals

import rpm
import os

from dnf.i18n import ucd


def checkSig(ts, package):
    """Takes a transaction set and a package, check it's sigs,
    return 0 if they are all fine
    return 1 if the gpg key can't be found
    return 2 if the header is in someway damaged
    return 3 if the key is not trusted
    return 4 if the pkg is not gpg or pgp signed"""

    value = 0
    currentflags = ts.setVSFlags(0)
    fdno = os.open(package, os.O_RDONLY)
    try:
        hdr = ts.hdrFromFdno(fdno)
    except rpm.error as e:
        if str(e) == "public key not available":
            value = 1
        if str(e) == "public key not trusted":
            value = 3
        if str(e) == "error reading package header":
            value = 2
    else:
        # checks signature from an hdr
        string = '%|DSAHEADER?{%{DSAHEADER:pgpsig}}:{%|RSAHEADER?{%{RSAHEADER:pgpsig}}:' \
                 '{%|SIGGPG?{%{SIGGPG:pgpsig}}:{%|SIGPGP?{%{SIGPGP:pgpsig}}:{(none)}|}|}|}|'
        try:
            siginfo = hdr.sprintf(string)
            siginfo = ucd(siginfo)
            if siginfo == '(none)':
                value = 4
        except UnicodeDecodeError:
            pass

        del hdr

    try:
        os.close(fdno)
    except OSError as e:  # if we're not opened, don't scream about it
        pass

    ts.setVSFlags(currentflags)  # put things back like they were before
    return value



# -*- coding: utf-8 -*-

# Copyright (C) 2014-2018 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#

from __future__ import absolute_import

import dnf.cli.commands.search as search
import dnf.match_counter
import dnf.pycomp

import tests.support
from tests import mock


class SearchCountedTest(tests.support.DnfBaseTestCase):

    REPOS = ["main"]
    CLI = "mock"

    def setUp(self):
        super(SearchCountedTest, self).setUp()
        self.cmd = search.SearchCommand(self.cli)

    def test_search_counted(self):
        counter = dnf.match_counter.MatchCounter()
        self.cmd._search_counted(counter, 'summary', 'ation')
        self.assertEqual(len(counter), 2)
        haystacks = set()
        for pkg in counter:
            haystacks.update(counter.matched_haystacks(pkg))
        self.assertCountEqual(haystacks, ["It's an invitation.",
                                          "Make a reservation."])

    def test_search_counted_glob(self):
        counter = dnf.match_counter.MatchCounter()
        self.cmd._search_counted(counter, 'summary', '*invit*')
        self.assertEqual(len(counter), 1)


class SearchTest(tests.support.DnfBaseTestCase):

    REPOS = ["search"]
    CLI = "mock"

    def setUp(self):
        super(SearchTest, self).setUp()
        self.base.output = mock.MagicMock()
        self.base.output.fmtSection = lambda str: str
        self.cmd = search.SearchCommand(self.cli)

    def patched_search(self, *args):
        with tests.support.patch_std_streams() as (stdout, _):
            tests.support.command_run(self.cmd, *args)
            call_args = self.base.output.matchcallback.call_args_list
            pkgs = [c[0][0] for c in call_args]
            return (stdout.getvalue(), pkgs)

    def test_search(self):
        (_, pkgs) = self.patched_search(['lotus'])
        pkg_names = list(map(str, pkgs))
        self.assertIn('lotus-3-16.i686', pkg_names)
        self.assertIn('lotus-3-16.x86_64', pkg_names)

    @mock.patch('dnf.cli.commands.search._',
                dnf.pycomp.NullTranslations().ugettext)
    def test_search_caseness(self):
        (stdout, pkgs) = self.patched_search(['LOTUS'])
        self.assertEqual(stdout, 'Name Matched: LOTUS\n')
        pkg_names = map(str, pkgs)
        self.assertIn('lotus-3-16.i686', pkg_names)
        self.assertIn('lotus-3-16.x86_64', pkg_names)



# -*- coding: utf-8 -*-


from __future__ import absolute_import
from __future__ import unicode_literals

import dnf
import dnf.cli.cli

from .common import TestCase


class DnfCliCliApiTest(TestCase):
    def setUp(self):
        base = dnf.Base(dnf.conf.Conf())
        self.cli = dnf.cli.cli.Cli(base=base)

    def test_cli(self):
        # dnf.cli.cli.Cli
        self.assertHasAttr(dnf.cli.cli, "Cli")
        self.assertHasType(dnf.cli.cli.Cli, object)

    def test_init(self):
        base = dnf.Base(dnf.conf.Conf())
        _ = dnf.cli.cli.Cli(base=base)

    def test_demands(self):
        # dnf.cli.cli.Cli.demands
        self.assertHasAttr(self.cli, "demands")
        self.assertHasType(self.cli.demands, dnf.cli.demand.DemandSheet)

    def test_redirect_logger(self):
        # dnf.cli.cli.Cli.redirect_logger
        self.assertHasAttr(self.cli, "redirect_logger")
        self.cli.redirect_logger(stdout=None, stderr=None)

    def test_register_command(self):
        # dnf.cli.cli.Cli.register_command
        self.assertHasAttr(self.cli, "register_command")
        command_cls = dnf.cli.commands.Command(cli=self.cli)
        self.cli.register_command(command_cls=command_cls)



# -*- coding: utf-8 -*-

# Copyright (C) 2014-2018 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#

from __future__ import absolute_import
from __future__ import unicode_literals

import libdnf.transaction

import dnf.goal
import dnf.selector

import tests.support


class GoalTest(tests.support.DnfBaseTestCase):

    REPOS = ['main']
    INIT_SACK = True

    def test_get_reason(self):
        sltr = dnf.selector.Selector(self.sack)
        sltr.set(name='mrkite')
        grp_sltr = dnf.selector.Selector(self.sack)
        grp_sltr.set(name='lotus')

        self.goal.install(select=sltr)
        self.goal.install(select=grp_sltr)
        self.goal.group_members.add('lotus')
        self.goal.run()
        installs = self.goal.list_installs()
        mrkite = [pkg for pkg in installs if pkg.name == 'mrkite'][0]
        lotus = [pkg for pkg in installs if pkg.name == 'lotus'][0]
        trampoline = [pkg for pkg in installs if pkg.name == 'trampoline'][0]
        self.assertEqual(self.goal.get_reason(lotus), libdnf.transaction.TransactionItemReason_GROUP)
        self.assertEqual(self.goal.get_reason(mrkite), libdnf.transaction.TransactionItemReason_USER)
        self.assertEqual(self.goal.get_reason(trampoline), libdnf.transaction.TransactionItemReason_DEPENDENCY)

    def test_group_reason(self):
        hole = self.sack.query().filter(name='hole')[0]
        self.goal.group_members.add('hole')
        self.assertEqual(libdnf.transaction.TransactionItemReason_GROUP, self.goal.group_reason(hole, libdnf.transaction.TransactionItemReason_GROUP))
        self.assertEqual(libdnf.transaction.TransactionItemReason_DEPENDENCY, self.goal.group_reason(hole, libdnf.transaction.TransactionItemReason_DEPENDENCY))



# -*- coding: utf-8 -*-


from __future__ import absolute_import
from __future__ import unicode_literals

import dnf

import types
from .common import TestCase


class DnfRepodictApiTest(TestCase):
    def setUp(self):
        self.repodict = dnf.repodict.RepoDict()

    def test_init(self):
        _ = dnf.repodict.RepoDict()

    def test_add(self):
        # RepoDict.add
        self.assertHasAttr(self.repodict, "add")
        self.repodict.add(repo=dnf.repo.Repo())

    def test_all(self):
        # RepoDict.all
        self.assertHasAttr(self.repodict, "all")
        self.assertHasType(self.repodict.all(), list)

    def test_add_new_repo(self):
        # RepoDict.add_new_repo
        self.assertHasAttr(self.repodict, "add_new_repo")
        self.assertHasType(
            self.repodict.add_new_repo(
                repoid="r",
                conf=dnf.conf.Conf(),
                baseurl=(""),
                key1="val1",
                key2="val2"
            ), dnf.repo.Repo)

    def test_enable_debug_repos(self):
        # RepoDict.enable_debug_repos
        self.assertHasAttr(self.repodict, "enable_debug_repos")
        self.repodict.enable_debug_repos()

    def test_enable_source_repos(self):
        # RepoDict.enable_source_repos
        self.assertHasAttr(self.repodict, "enable_source_repos")
        self.repodict.enable_source_repos()

    def test_get_matching(self):
        # RepoDict.get_matching
        self.assertHasAttr(self.repodict, "get_matching")
        self.assertHasType(self.repodict.get_matching(key=""), list)

    def test_iter_enabled(self):
        # RepoDict.iter_enabled
        self.assertHasAttr(self.repodict, "iter_enabled")
        self.assertHasType(self.repodict.iter_enabled(), types.GeneratorType)



# -*- coding: utf-8 -*-


from __future__ import absolute_import
from __future__ import unicode_literals

import dnf

from .common import TestCase


class DnfRpmApiTest(TestCase):
    def test_detect_releasever(self):
        # dnf.rpm.detect_releasever
        self.assertHasAttr(dnf.rpm, "detect_releasever")

    def test_basearch(self):
        # dnf.rpm.basearch
        self.assertHasAttr(dnf.rpm, "basearch")
        self.assertHasType(dnf.rpm.basearch(arch="x86_64"), str)



# -*- coding: utf-8 -*-


from __future__ import absolute_import
from __future__ import unicode_literals

import unittest

import dnf

from .common import TestCase


class DnfApiTest(TestCase):
    def test_version(self):
        # dnf.__version__
        self.assertHasAttr(dnf, "__version__")
        self.assertHasType(dnf.__version__, str)

    def test_base(self):
        # dnf.Base
        self.assertHasAttr(dnf, "Base")
        self.assertHasType(dnf.Base, object)

    def test_plugin(self):
        # dnf.Plugin
        self.assertHasAttr(dnf, "Plugin")
        self.assertHasType(dnf.Plugin, object)



# -*- coding: utf-8 -*-

# Copyright (C) 2012-2018 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.

from __future__ import absolute_import
from __future__ import unicode_literals

import dnf.callback
import dnf.cli.progress
import dnf.pycomp

import tests.support
from tests.support import mock


class MockStdout(dnf.pycomp.StringIO):
    def visible_lines(self):
        lines = self.lines()
        last = len(lines) - 1
        return [l[:-1] for (i, l) in enumerate(lines)
                if l.endswith('\n') or i == last]

    def lines(self):
        return self.getvalue().splitlines(True)


class FakePayload(object):
    def __init__(self, string, size):
        self.string = string
        self._size = size

    def __str__(self):
        return self.string

    @property
    def download_size(self):
        return self._size


class ProgressTest(tests.support.TestCase):
    def test_single(self):
        now = 1379406823.9
        fo = MockStdout()
        with mock.patch('dnf.cli.progress._term_width', return_value=60), \
                mock.patch('dnf.cli.progress.time', lambda: now):

            p = dnf.cli.progress.MultiFileProgressMeter(fo)
            p.isatty = True
            pload = FakePayload('dummy-text', 5)
            p.start(1, 1)
            for i in range(6):
                now += 1.0
                p.progress(pload, i)
            p.end(pload, None, None)
        self.assertEqual(fo.lines(), [
            'dummy-text  0% [          ] ---  B/s |   0  B     --:-- ETA\r',
            'dummy-text 20% [==        ] 1.0  B/s |   1  B     00:04 ETA\r',
            'dummy-text 40% [====      ] 1.0  B/s |   2  B     00:03 ETA\r',
            'dummy-text 60% [======    ] 1.0  B/s |   3  B     00:02 ETA\r',
            'dummy-text 80% [========  ] 1.0  B/s |   4  B     00:01 ETA\r',
            'dummy-text100% [==========] 1.0  B/s |   5  B     00:00 ETA\r',
            'dummy-text                  1.0  B/s |   5  B     00:05    \n'])

    def test_mirror(self):
        fo = MockStdout()
        p = dnf.cli.progress.MultiFileProgressMeter(fo, update_period=-1)
        p.isatty = True
        p.start(1, 5)
        pload = FakePayload('foo', 5.0)
        now = 1379406823.9

        with mock.patch('dnf.cli.progress._term_width', return_value=60), \
                mock.patch('dnf.cli.progress.time', lambda: now):
            p.progress(pload, 3)
            p.end(pload, dnf.callback.STATUS_MIRROR, 'Timeout.')
            p.progress(pload, 4)
        self.assertEqual(fo.visible_lines(), [
            '[MIRROR] foo: Timeout.                                     ',
            'foo        80% [========  ] ---  B/s |   4  B     --:-- ETA'])

    _REFERENCE_TAB = [
        ['(1-2/2): f  0% [          ] ---  B/s |   0  B     --:-- ETA'],
        ['(1-2/2): b 10% [=         ] 2.2  B/s |   3  B     00:12 ETA'],
        ['(1-2/2): f 20% [==        ] 2.4  B/s |   6  B     00:10 ETA'],
        ['(1-2/2): b 30% [===       ] 2.5  B/s |   9  B     00:08 ETA'],
        ['(1-2/2): f 40% [====      ] 2.6  B/s |  12  B     00:06 ETA'],
        ['(1-2/2): b 50% [=====     ] 2.7  B/s |  15  B     00:05 ETA'],
        ['(1-2/2): f 60% [======    ] 2.8  B/s |  18  B     00:04 ETA'],
        ['(1-2/2): b 70% [=======   ] 2.8  B/s |  21  B     00:03 ETA'],
        ['(1-2/2): f 80% [========  ] 2.9  B/s |  24  B     00:02 ETA'],
        ['(1-2/2): b 90% [========= ] 2.9  B/s |  27  B     00:01 ETA'],
        ['(1/2): foo                  1.0  B/s |  10  B     00:10    ',
         '(2/2): bar100% [==========] 2.9  B/s |  30  B     00:00 ETA']]

    def test_multi(self):
        now = 1379406823.9
        fo = MockStdout()
        with mock.patch('dnf.cli.progress._term_width', return_value=60), \
                mock.patch('dnf.cli.progress.time', lambda: now):

            p = dnf.cli.progress.MultiFileProgressMeter(fo)
            p.isatty = True
            p.start(2, 30)
            pload1 = FakePayload('foo', 10.0)
            pload2 = FakePayload('bar', 20.0)
            for i in range(11):
                p.progress(pload1, float(i))
                if i == 10:
                    p.end(pload1, None, None)
                now += 0.5

                p.progress(pload2, float(i * 2))
                self.assertEqual(self._REFERENCE_TAB[i], fo.visible_lines())
                if i == 10:
                    p.end(pload2, dnf.callback.STATUS_FAILED, 'some error')
                now += 0.5

        # check "end" events
        self.assertEqual(fo.visible_lines(), [
            '(1/2): foo                  1.0  B/s |  10  B     00:10    ',
            '[FAILED] bar: some error                                   '])
        self.assertTrue(2.0 < p.rate < 4.0)

    @mock.patch('dnf.cli.progress._term_width', return_value=40)
    def test_skip(self, mock_term_width):
        fo = MockStdout()
        p = dnf.cli.progress.MultiFileProgressMeter(fo)
        p.start(2, 30)
        pload1 = FakePayload('club', 20.0)
        p.end(pload1, dnf.callback.STATUS_ALREADY_EXISTS, 'already got')
        self.assertEqual(p.done_files, 1)
        self.assertEqual(p.done_size, pload1._size)
        self.assertEqual(fo.getvalue(),
                         '[SKIPPED] club: already got            \n')
